#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# changeUserPass.pl changes a linux user password only if they are in CSMuser
#
# Chi D Nguyen <cdn@us.ibm.com> (5 October 1999)
#

#############################################
# First we have our configuration variables #
#############################################

use ASM::Common;
commonInit(["userid=s","password=s","oldpassword=s"],
	   "--userid=CSMUSERID --password=PASSWORD [ --oldpassword=OLDPASSWORD ]");

use ASM::Exec;
use User::grent;

setFacility('sysuser');

if($opt_help || !$opt_userid || !$opt_password) {
  logUsage();
  exit(3);
}
else {
# get the user that we want to change the password of

  $pw = getpwnam $opt_userid; 

# Check to see if a real user.

  if (!$pw) {
    error(MSG_NO_SUCH_USER, $opt_userid);
    exit 5;
  }

# Check to see if in CSMuser group

  my $gr = getgr($CSMgrp);
  if ($pw->gid != $gr->gid) {
    error(MSG_NOT_ASM_USER, $opt_userid);
    exit 6;
  }

# This is definitely a CSM user.


  if ($opt_oldpassword) { 

# Check to see if the old password is correct.
# We are using shadow passwords here so this is a kludge.

    openOrDie(PASSWDFILE,"/etc/shadow");

    while ($line = <PASSWDFILE>) {
      ($userid,$userpass,$remainder) = split /:/,$line,3; # get password
      if ($userid eq $pw->name) {  # found the password
	last;
      }
    }

    close(PASSWDFILE);
 
    if(crypt($opt_oldpassword, $userpass) ne $userpass){
      trace("old password is incorrect");
      error(MSG_E10);		# password was incorrect
      exit 3;
    }
  }

# Change the password.

  my ($rc, $stdout, $stderr);
  trace("changing password for $opt_userid using {echo $opt_userid:*** | chpasswd}.");
  $rc = runit_notrace(["sh", "-c", "echo $opt_userid:$opt_password | chpasswd "],
	      \$stdout, \$stderr);
  trace("changing password for $opt_userid...rc=$rc");
  if ($rc == 0) {
    audit(MSG_USER_PASSWORD_CHANGED, $opt_userid);
  }
  exit $rc;
}
