#!/usr/bin/perl
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
# checkWeb.pl <target>
#
# Checks up on the web server at <target> and determines if it is
# functioning. This is done by invoking the extract script at port :1959
# and parsing the results of that call.
#
# Jeff Cates, <jcates@us.ibm.com> (21 October, 1999)
#     (minor modification by Wayne Gao <waynegao@us.ibm.com> 10/22/1999)
#

use strict;

my ($rc, $eSP_BIN, $target, $savefile, $serverok, $done, $loop, $delay, $match, $error);

use ASM::Common;

trace("+++ Start checkWeb.pl");

$eSP_BIN='/opt/CSM/scripts';
require "$eSP_BIN/extract.pl";

# Determine input values
if (@ARGV == 0) { $target = ":1959"; }
else            { $target = $ARGV[0]; }

# if (@ARGV >  1) { $savefile = $ARGV[1]; }
# else            { $savefile = "temp"; }
#$target = ":1959";        # this is the url value passed to extract
$savefile = "temp";       # data file for extract to return data

$serverok = 1;            # final return 0 if OK and 1 otherwise
$done = 0;                # control completion
$loop = 5;                # loop control for socket errors
$delay = 2;               # delay in seconds for looping
$match = "International Business Machines"; # match string in returned page
$error = 0;               # keep up with errors


while (!$done) {
  $rc = &extract($target, $savefile); # Get web server status to $savefile

  # Open results file and check it out
  if (!$rc && open(WRES, $savefile)) {
    while (<WRES>) {
      # ERROR message from extract.pl
      if ($_ =~ /^ERROR -/) {
        print $_;
        $loop--;
        trace "Monitor will attempt to connect $loop more times.\n";
        if ($loop) { sleep($delay); }
        else { $error = 1; $done = 1; }
      }
      # Otherwise look for match string
      elsif ($_ =~ /$match/) {
        trace "Web server is up and running !!\n";
        $done = 1;
        $serverok = 0;
      }
    } # end while WRES
    # Page is parsed and still not done, so there was not match.
    if (!$error && $serverok) {
      trace "Page retrieved but no match on: $match.\n";
      $done = 1;
      $error = 2;
    }
  } # end result file opened
  else {
    trace "Can't open $savefile: $!\n";
    $loop--;
    trace "Monitor will attempt to connect $loop more times.\n";
    if ($loop) { sleep($delay); }
    else { $done = 1; }
  }

} # end while not done

trace "Return value is $serverok\n";
exit ($serverok);
