#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# ck-app.pl - run after an app upgrade, see if upgrade was successful
#

use strict;
my ($eSP_BIN, $RLOGF, $un_inst, $monitor, $status, $date, $pkg_name, $cnt, $stdout, $stderr, $rc);

$eSP_BIN = "/opt/CSM/scripts";

use ASM::Common;
require "$eSP_BIN/common_install.pl";

setFacility('initsys');

use vars qw! $setup_data_file $IMG_DIR !;

commonInit([],"");

$RLOGF = "/tmp/new-app-reboot.log";

$un_inst = "$eSP_BIN/uninstallImage.pl";
$monitor = "$eSP_BIN/monitorASM.pl";
$status = 1;

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;

   trace("$exitmsg");
   trace("exit $exitrc");
   $date = `date|tr -d "\n"`;
   trace("++++ Stop: ck_app.pl ($date) ++++");
   exit $exitrc;

}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}                                          

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

chop($pkg_name = `grep "PkgName" $IMG_DIR/pkgInfo.txt|grep -v "\#"|cut -f2 -d "="|tr -d " "`);

$date = `date|tr -d "\n"`;
trace("++++ Start: ck_app.pl ($date) ++++");

if (! -e "$RLOGF")
{
   #  App log is set to 1 in installImage.pl for App updates.
   audit(MSG_SYSTEM_ACTIVE);
   &cleanup_exit("Application upgrade only, don't check system\n",0);
}

# Give everything time to start
sleep 30;

# At this point, we know the upgrade involves app upgrade.
# Run monitorASM.pl to check status of the system.

if (-e "$monitor")
{
   trace("Run: $monitor");
   $status = runit(["$monitor"], \$stdout, \$stderr);
}

if ($status != 0)
{
   chop($cnt = `cat $RLOGF`);
   $cnt = $cnt + 1;
   $rc = runit(["echo $cnt > $RLOGF"], \$stdout, \$stderr);
   if ($cnt <= 2)
   {
      error(MSG_SYSTEM_FAILED_INIT);
      trace("New app has problem, reboot with the new app again");
      trace("shutdown -r now");
      $rc = runit(["shutdown -r now"], \$stdout, \$stderr);
   }
   else
   {
      error(MSG_SYSTEM_FAILED_INIT_RESTART);
      trace("Rebooted new app twice, roll back to the old app and un-install application upgrade.");
      trace("Upgrade failed, reboot the machine to roll back old app.\n");
      set_failure_flag();

      $rc = runit(["$un_inst --pkg_name=$pkg_name"], \$stdout, \$stderr);
      if ($rc != 0)
      {
         trace("Uninstall was not successfull.   rc = $rc");
      }
      $rc = runit(["rm -f $RLOGF"], \$stdout, \$stderr);
      trace("shutdown -r now");
      $rc = runit(["shutdown -r now"], \$stdout, \$stderr);
   } 
}
else
{
   audit(MSG_SYSTEM_ACTIVE);
   trace("Reboot new app successfully.");
   trace(`date`);
   trace("Reboot successfully, remove the flag file.");
   trace("Upgrade installed successfully.");
   trace("rm -f $RLOGF");
   $rc = runit(["rm -f $RLOGF"], \$stdout, \$stderr);
   $rc = runit(["uname -a"], \$stdout, \$stderr);
}

&cleanup_exit("", 0);


# Set "upgradeFailed=1" in csm.data
sub set_failure_flag {
  trace("Setting failure flag");
  local *IN;
  local *OUT;
  if (!open(IN, "<$setup_data_file")) {
    error(MSG_COULD_NOT_OPEN_FILE, $setup_data_file);
    return;
  }
  my @lines = <IN>;
  close(IN);
  my $found = 0;
  foreach my $line (@lines) {
    if ($line =~ /upgradeFailed/ && !$found) {
      $line = "upgradeFailed=1\n";
      $found = 1;
    }
  }
  if (!$found) {
    push(@lines, "upgradeFailed=1\n");
  }
  if (!open(OUT, ">$setup_data_file")) {
    error(MSG_COULD_NOT_OPEN_FILE, $setup_data_file);
    return;
  }
  print OUT @lines;
  close(OUT);
}
