#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# ck-kernel.pl - run after a kernel upgrade, modify lilo.conf file
#
# Wayne Gao, <waynegao@us.ibm.com> (29 Octobe 1999)
#
use strict;
my ($eSP_BIN, $RLOGF, $un_inst, $monitor, $status, $date, $pkg_name, $cnt, $r_knel, $k_vsion, $stdout, $stderr, $rc);

$eSP_BIN = "/opt/CSM/scripts";

use ASM::Common;
use ASM::Exec;
use ASM::Log;

require "$eSP_BIN/common_install.pl";

setFacility('initsys');

use vars qw! $setup_data_file $IMG_DIR !;

commonInit([],"");

$RLOGF = "/tmp/new-kernel-reboot.log";

$un_inst = "$eSP_BIN/uninstallImage.pl";
$monitor = "$eSP_BIN/monitorASM.pl";
$status = 0;

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;

   trace("$exitmsg");
   trace("exit $exitrc");
   $date = `date|tr -d "\n"`;
   trace("++++ Stop: ck_kernel.pl ($date) ++++");
   exit $exitrc;

}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}                                          

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

chop($pkg_name = `grep "PkgName" $IMG_DIR/pkgInfo.txt|grep -v "\#"|cut -f2 -d "="|tr -d " "`);

chop($k_vsion = `grep "KernelVersion" $IMG_DIR/pkgInfo.txt|grep -v "\#"|cut -f2 -d "="|tr -d " "`);

$date = `date|tr -d "\n"`;
trace("++++ Start: ck_kernel.pl ($date) ++++");

if (! -e "$RLOGF")
{
   #  Kernel log is set to 1 in installImage.pl for Kernel updates.
   audit(MSG_SYSTEM_ACTIVE);
   &cleanup_exit("Application upgrade only, don't check system\n",0);
}

sleep 10;

# At this point, we know the upgrade involves kernel upgrade.
# Run monitorASM.pl to check status of the system.

if (-e "$monitor")
{
   trace("Run: $monitor");
   $status = runit(["$monitor"], \$stdout, \$stderr);
}

# Check to see if the running kernel version is equal to the new kernel version, 
# else the new kernel failed to load.  @change by cdn

chop($r_knel = `uname -r`);
if ($k_vsion ne  $r_knel)
{
  $status = 1;
}

if ($status != 0)
{
   chop($cnt = `cat $RLOGF`);
   $cnt = $cnt + 1;
   $rc = runit(["echo $cnt > $RLOGF"], \$stdout, \$stderr);
   if ($cnt <= 2)
   {
      error(MSG_SYSTEM_FAILED_INIT, $cnt);
      trace("New kernel had a problem, reboot with the new kernel again");
      trace(`uname -a`);
      $rc = runit(["/sbin/lilo"], \$stdout, \$stderr);
      trace("shutdown -r now");
      $rc = runit(["shutdown -r now"], \$stdout, \$stderr);
   }
   else
   {
      error(MSG_SYSTEM_FAILED_INIT_RESTART);
      trace("Rebooted new kernel twice, roll back to the old kernel and un-install software upgrade.");
      trace("Upgrade failed, reboot the machine to roll back old kernel.");
      set_failure_flag();

      $rc = runit(["$un_inst --pkg_name=$pkg_name"], \$stdout, \$stderr);
      $rc = runit(["rm -f $RLOGF"], \$stdout, \$stderr);
      trace(`uname -a`);
      trace("shutdown -r now");
      $rc = runit(["shutdown -r now"], \$stdout, \$stderr);
   } 
}
else
{
   audit(MSG_SYSTEM_ACTIVE);
   trace("Reboot new kernel successfully.");
   trace(`date`);
   trace("Reboot successfully, remove the flag file.");
   trace("Upgrade installed successfully.");
   trace("rm -f $RLOGF");
   $rc = runit(["rm -f $RLOGF"], \$stdout, \$stderr);
   $rc = runit(["uname -a"], \$stdout, \$stderr);
}

&cleanup_exit("", 0);


# Set "upgradeFailed=1" in csm.data
sub set_failure_flag {
  local *IN;
  local *OUT;
  if (!open(IN, "<$setup_data_file")) {
    error(MSG_COULD_NOT_OPEN_FILE, $setup_data_file);
    return;
  }
  my @lines = <IN>;
  close(IN);
  my $found = 0;
  foreach my $line (@lines) {
    if ($line =~ /upgradeFailed/ && !$found) {
      $line = "upgradeFailed=1\n";
      $found = 1;
    }
  }
  if (!$found) {
    push(@lines, "upgradeFailed=1\n");
  }
  if (!open(OUT, ">$setup_data_file")) {
    error(MSG_COULD_NOT_OPEN_FILE, $setup_data_file);;
    return;
  }
  print OUT @lines;
  close(OUT);
}
