#!/usr/bin/perl -U -w
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


# cleanTempLogFiles.pl removes tmp files from /tmp left by
# the ASM log viewing functions after they have not been
# accessed for a period of time (10 minutes).
# It should be invoked every so often from cron.
#
# Dan Poirier <poirier@us.ibm.com> (27 September 2000)
#

# Call with --install to add to crontab
# Call with --uninstall to remove from crontab


# Modules to use
use File::Find;
use strict;
use POSIX 'setsid';

use ASM::Common;
commonInit(["daemon", "stop"],
	   "[--daemon | --stop]");
setFacility('log');

use vars qw(  $opt_daemon $opt_stop);

(my $me = $0) =~ s|^.*/||;

my $filename = "/opt/CSM/scripts/cleanTempLogFiles.pl";
my $pid_file = "/opt/CSM/data/cleanTempLogFiles.pid";
my $name = "cleanTempLogFiles";	# name to look for in ps
my $DEBUG = 0;

if ($opt_stop) {
  signal_stop();
  exit(0);
}
if ($opt_daemon) {
  if (alreadyRunning($pid_file, $name)) {
    print STDERR "$me: already running\n";
    exit(0);
  } else {
    $SIG{TERM} = \&cleanup_and_exit;
    daemonize($pid_file);
  }
}

# Time in minutes after last modification before we can
# delete the file
my $maxMinutes = 5;

# Same, in days
my $maxDays = $maxMinutes / (24.0 * 60.0) ;

while (1) {
  find(\&wanted, "/tmp");
  exit(0) unless $opt_daemon;
  sleep(60);
}

# This gets called for each file that File::Find looks at
sub wanted {
  # We are chdirred to the directory of the current file
  # The current filename is $_
  # $File::Find::name is the complete pathname

  # Don't try to delete directories, links, sockets, etc...
  return unless -f;
  
  # Only match filesname starting with csmlog
  return unless /^csmlog/;

  # Get time since last mod, in DAYS :-)
  my $modtime = -M;
  if ($modtime > $maxDays) {
    trace "Deleting $File::Find::name\n";
    unlink($File::Find::name) or trace "Error deleting $File::Find::name: $!\n";
  }
}

sub send_signal($) {
  my ($signal) = @_;
  # Send a signal to a running daemon to reload its variables or quit
  local(*IN);
  if (open(IN, "<$pid_file")) {
    chomp(my $pid = <IN>);
    close(IN);
    kill $signal, $pid;
  }
}

sub signal_stop {
  send_signal(15);
}

sub cleanup_and_exit {
  if ($opt_daemon) {
    unlink($pid_file);
  }
  exit(0);
}

