#
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


#
# common_mailmgr.pl - cfg variable and common functions used by the Mail List Mgr ASM scripts.
#

require ASM::Exec;
require ASM::Common;

$MAILMGR_PATH = "/usr/lib/majordomo";
$MAILMGR_HOME = "/var/lib/majordomo";
$MAILMGR_LIST_PATH = "/var/lib/majordomo";
$MAILMGR_LIST_DIR = "/var/lib/majordomo/lists";
$MAILMGR_ALIASES = "$MAILMGR_PATH/majordomo.aliases";
$MAILMGR_ALIASES_DB = "$MAILMGR_PATH/majordomo.aliases.db";
$MAILMGR_CF = "/etc/majordomo.cf";
$MAILMGR_WRAPPER = "$MAILMGR_PATH/wrapper";
$SECURE_WRAPPER = "/etc/smrsh/wrapper";

#  Use Majordomo perl routines to handle list commands
unshift(@INC, $MAILMGR_PATH);
require "majordomo.pl";
require "shlock.pl";
require "config_parse.pl";

sub create_maillist_config
{
   my ($name) = @_;

   #   Creates the config file when it doesn't exist.
   &get_config($MAILMGR_LIST_DIR, $name);
   $rc = runit(["/bin/chown majordomo $MAILMGR_LIST_DIR/$name.config"], \$stdout, \$stderr);
   $rc = runit(["/bin/chgrp majordomo $MAILMGR_LIST_DIR/$name.config"], \$stdout, \$stderr);

   return(0);

}

sub get_maillists
{
   my (@dirfiles, $file, $listname, $addrlist);
   my (@lists);

   opendir DIR, "$MAILMGR_LIST_DIR";
   @lists = ();
   @dirfiles = readdir DIR;
   foreach $file (@dirfiles)
   {
      chomp($file);
      $_ = $file;
      if (/^[a-zA-Z0-9_-]+/)
      {
         #  Each valid list should have a config file.
         if (-e "$MAILMGR_LIST_DIR/$_.config")
         {
            $listname = $_;
            push (@lists, $listname);
         }
      }
   }
   return(@lists);
}

sub update_maillist_files
{
   my ($name, $action) = @_;
   if ($action eq "add")
   {
      #  See if the name already exists in majordomo
      if (&valid_list($MAILMGR_LIST_DIR, $name) ne "")
      {
         error(MSG_MAILLIST_NAME_EXISTS, $name);
         trace("Majordomo indicates the mail list name $name already exists.\n");
         return(1);
      }
      $rc = runit(["/bin/touch $MAILMGR_LIST_DIR/$name"], \$stdout, \$stderr);
      $rc = runit(["/bin/touch $MAILMGR_LIST_DIR/$name.info"], \$stdout, \$stderr);
      $rc = runit(["/bin/chown majordomo $MAILMGR_LIST_DIR/$name"], \$stdout, \$stderr);
      $rc = runit(["/bin/chgrp majordomo $MAILMGR_LIST_DIR/$name"], \$stdout, \$stderr);
      $rc = runit(["/bin/chown majordomo $MAILMGR_LIST_DIR/$name.info"], \$stdout, \$stderr);
      $rc = runit(["/bin/chgrp majordomo $MAILMGR_LIST_DIR/$name.info"], \$stdout, \$stderr);
   }
   elsif ($action eq "delete")
   {
      $rc = runit(["/bin/rm -f $MAILMGR_LIST_DIR/$name"], \$stdout, \$stderr);
      $rc = runit(["/bin/rm -f $MAILMGR_LIST_DIR/$name.info"], \$stdout, \$stderr);
      $rc = runit(["/bin/rm -f $MAILMGR_LIST_DIR/$name.config"], \$stdout, \$stderr);
   }

   return(0);

}

sub create_alias_maillist
{
   my ($owner) = @_;

   openOrDie(OUT, "> $MAILMGR_ALIASES");
   print OUT "# majordomo.aliases created by $0.\n\n";
   print OUT "majordomo: \"|/usr/lib/majordomo/wrapper majordomo\"\n";
   print OUT "owner-majordomo: $owner\n";
   print OUT "majordomo-owner: $owner\n";
   close(OUT);
   
   $rc = runit(["/bin/touch $MAILMGR_ALIASES_DB"], \$stdout, \$stderr);
   if ($rc != 0)
   {
      error(MSG_MAILMGR_ALIAS_ERROR);
      trace("Errors creating $MAILMGR_ALIAS_DB file: $stdout $stderr\n");
   }

   $rc = runit(["/bin/chmod 0644 $MAILMGR_ALIASES"], \$stdout, \$stderr);
   $rc = runit(["/bin/chmod 0644 $MAILMGR_ALIASES_DB"], \$stdout, \$stderr);

   # Make modifications to Mail List Manager aliases take effect
   $rc = runit(["/usr/bin/newaliases"], \$stdout, \$stderr);
   if ($rc != 0)
   {
      error(MSG_MAILMGR_ALIAS_ERROR);
      trace("Errors running newaliases: $stdout $stderr\n");
   }
   return($rc);
}

sub set_admin_maillist
{
   my ($name) = @_;
   my (@aliases);

   openOrDie(ALIAS, "< $MAILMGR_ALIASES");
   @aliases = ();
   while (<ALIAS>)
   {
         if (/^\s*majordomo-owner:/)
         {
            push(@aliases, "majordomo-owner:	$name\n");
         }
         elsif (/^\s*owner-majordomo:/)
         {
            push(@aliases, "owner-majordomo:	$name\n");
         }
         else
         {
            push(@aliases, $_);
         }
   }
   close(ALIAS);
   openOrDie(OUT, "> $MAILMGR_ALIASES");
   print OUT @aliases;
   close(OUT);
   
   # Make modifications to Mail List Manager aliases take effect
   $rc = runit(["/usr/bin/newaliases"], \$stdout, \$stderr);
   if ($rc != 0)
   {
     error(MSG_MAILMGR_ALIAS_ERROR);
     trace("Errors running newaliases: $stdout $stderr\n");
     return($rc);
   }
   return(0);
}

sub get_admin_maillist
{
   my ($name) = "owner-majordomo";
   my ($adminid);

   $adminid = "";
   openOrDie(ALIAS, "< $MAILMGR_ALIASES");
   while (<ALIAS>)
   {
      if (/^\s*$name:\s*(.*)/)
      {
         $adminid = $1;
         return($adminid);
      }
   }
   close(ALIAS);
   return($adminid);
}

sub remove_alias_maillist
{
   my ($owner) = @_;

   renameOrDie($MAILMGR_ALIASES, "$MAILMGR_ALIASES.save");
   
   # Make modifications to Mail List Manager aliases take effect
   $rc = runit(["/usr/bin/newaliases"], \$stdout, \$stderr);
   if ($rc != 0)
   {
      error(MSG_MAILMGR_ALIAS_ERROR);
      trace("Errors running newaliases: $stdout $stderr\n");
   }
   return($rc);
}

sub update_alias_maillist
{
   my ($name, $action, $owner) = @_;
   my (@aliases, $already_added);

   openOrDie(ALIAS, "< $MAILMGR_ALIASES");
   while (<ALIAS>)
   {
      if (/^\s*$name:/)
      {
         $already_added = 1;
         if ($action eq "add")
         {
            push(@aliases, "$name:\t\"|$MAILMGR_WRAPPER resend -l $name $name-list\"\n");
            push(@aliases, "$name-list:\t:include:$MAILMGR_LIST_DIR/$name\n");
            push(@aliases, "owner-$name:\t$owner\n");
            push(@aliases, "$name-owner:\t$owner\n");
            push(@aliases, "$name-approval:\t$owner\n");
            push(@aliases, "$name-request:\t\"|$MAILMGR_WRAPPER majordomo -l $name\"\n");
         }
      }
      else
      {
         if (!(/^\s*$name-.*:/ || /^\s*.*-$name:/))
         {
            push(@aliases, $_);
         }
      }
   }
   if (!$already_added && ($action eq "add"))
   {
      push(@aliases, "$name:\t\"|$MAILMGR_WRAPPER resend -l $name $name-list\"\n");
      push(@aliases, "$name-list:\t:include:$MAILMGR_LIST_DIR/$name\n");
      push(@aliases, "owner-$name:\t$owner\n");
      push(@aliases, "$name-owner:\t$owner\n");
      push(@aliases, "$name-approval:\t$owner\n");
      push(@aliases, "$name-request:\t\"|$MAILMGR_WRAPPER majordomo -l $name\"\n");
   }
   close(ALIAS);
   openOrDie(OUT, "> $MAILMGR_ALIASES");
   print OUT @aliases;
   close(OUT);
   
   # Make modifications to Mail List Manager aliases take effect
   $rc = runit(["/usr/bin/newaliases"], \$stdout, \$stderr);
   if ($rc != 0)
   {
     error(MSG_MAILMGR_ALIAS_ERROR);
     trace("Errors running newaliases: $stdout $stderr\n");
     return($rc);
   }
   return(0);
}

sub get_maillist_config
{
   my ($name, $key) = @_;
   my ($value, @configlist, $found);

   $value = "";
   openOrDie(CONFIG, "< $MAILMGR_LIST_DIR/$name.config");
   $found = 0;
   while (<CONFIG>)
   {
      if (/^\s*$key\s*=\s*(.*)/)
      {
         $value = $1;
         $found = 1;
         last;
      }
      push(@configlist, $_);
   }
   close(CONFIG);
   if (!$found)
   {
      trace("Couldn't find the $key keyword in the config file $MAILMGR_LIST_DIR/$name.config");
   }
   return($value);
}

sub get_maillist_passwd
{
   my($name) = @_;

   #   Mail List's password defaults to admin_passwd value.
   return(&get_maillist_config($name, "admin_passwd"));
}

sub set_maillist_passwd
{
   my($name, $passwd) = @_;
   my($rc);

   $rc = 0;
   #   Mail List's password defaults to admin_passwd value.
   &update_maillist_config($name, "admin_passwd", $passwd);
   &update_maillist_config($name, "approve_passwd", $passwd);

   return($rc);
}

sub set_maillist_restricted
{
   my($name) = @_;
   my($rc);

   $rc = 0;

   &update_maillist_config($name, "restrict_post", "\t$MAILMGR_LIST_DIR/$opt_name");

   return($rc);
}

sub set_maillist_anypost
{
   my($name) = @_;
   my($rc);

   $rc = 0;

   &update_maillist_config($name, "restrict_post", "");

   return($rc);
}

sub get_maillist_anypost
{
   my($name) = @_;
   my($post);

   #   Posting is restricted to a list of emails in a file specified
   #   using the "restrict_post" keyword.  Posting is "open" if
   #   no file is specified.
   $post = &get_maillist_config($name, "restrict_post");
   if ($post ne "")
   {
      #  Posting is open to any email address.
      return(0);
   }
   else
   {
      #  Posting is restricted to a list of emails.
      return(1);
   }
}

sub set_maillist_subscribe
{
   my($name) = @_;
   my($rc);

   $rc = 0;

   &update_maillist_config($name, "subscribe_policy", "open");

   return($rc);
}

sub set_maillist_approve
{
   my($name) = @_;
   my($rc);

   $rc = 0;

   &update_maillist_config($name, "subscribe_policy", "closed");

   return($rc);
}

sub get_maillist_subscribe
{
   my($name) = @_;
   my($policy);

   #   Posting is restricted to a list of emails in a file specified
   #   using the "restrict_post" keyword.  Posting is "open" if
   #   no file is specified.
   $policy = &get_maillist_config($name, "subscribe_policy");
   if ($policy eq "open")
   {
      #  Subscriptions open to any email address.
      return(1);
   }
   elsif ($policy eq "closed")
   {
      #  Posting requests are approved.
      return(0);
   }
   else
   {
      #  Unsupported policy value
      trace("Mail list subscribe_policy value $policy is not supported.");
      return(0);
   }
}

sub update_maillist_config
{
   my ($name, $key, $value) = @_;
   my (@configlist, $found);

   $rc = runit(["/bin/cp", "-p", "$MAILMGR_LIST_DIR/$name.config", "$MAILMGR_LIST_DIR/$name.config.new"], \$stdout, \$stderr);
   openOrDie(CONFIG, "< $MAILMGR_LIST_DIR/$name.config");
   $found = 0;
   while (<CONFIG>)
   {
      if (/^\s*$key\s*=\s*(.*)/)
      {
         s/(^\s*$key\s*=\s*)(.*)\n/$1$value\n/;
         $found = 1;
      }
      push(@configlist, $_);
   }
   close(CONFIG);
   if (!$found)
   {
      push(@configlist, "$key\t=\t$value\n");
   }
   openOrDie(CONFIG, "> $MAILMGR_LIST_DIR/$name.config.new");
   print CONFIG @configlist;
   close(CONFIG);

   renameOrDie("$MAILMGR_LIST_DIR/$name.config.new", "$MAILMGR_LIST_DIR/$name.config");

   return(0);
}

sub edit_maillist_addr
{
   my ($name, $action, $addr, $newaddr) = @_;
   my (@aliases, $already_added, $rc);

   openOrDie(ADDRS, "< $MAILMGR_LIST_DIR/$name");
   $already_added = 0;
   while (<ADDRS>)
   {
      if (/^\s*$addr\s*/)
      {
         $already_added = 1;
         if (($action eq "add") || ($action ne "delete"))
         {
            #  Add checks to prevent addition of malicious aliases
            push(@aliases, "$newaddr\n");
         }
      }
   }
   if (!$already_added && ($action eq "add"))
   {
      #  Add checks to prevent addition of malicious aliases
      push(@aliases, "$newaddr\n");
   }
   close(ADDRS);
   openOrDie(OUT, "> $MAILMGR_LIST_DIR/$name");
   print OUT @aliases;
   close(OUT);
   
   # Make modifications to Mail List Manager aliases take effect
   $rc = runit(["/usr/bin/newaliases"], \$stdout, \$stderr);
   if ($rc != 0)
   {
     error(MSG_MAILMGR_ALIAS_ERROR);
     trace("Errors running newaliases: $stdout $stderr\n");
     return($rc);
   }
   return(0);
}

sub update_maillist_addrs
{
   my ($name, @newaddrs) = @_;
   my ($addr, $rc);

   openOrDie(ADDRS, "> $MAILMGR_LIST_DIR/$name");
   foreach $addr (@newaddrs)
   {
      #  Add checks to prevent addition of malicious aliases
      print ADDRS "$addr\n";
   }
   close(ADDRS);
   
   # Make modifications to Mail List Manager aliases take effect
   $rc = runit(["/usr/bin/newaliases"], \$stdout, \$stderr);
   if ($rc != 0)
   {
     error(MSG_MAILMGR_ALIAS_ERROR);
     trace("Errors running newaliases: $stdout $stderr\n");
     return($rc);
   }
   return(0);
}

sub update_maillist_addrfile
{
   my ($name, $newaddrfile) = @_;
   my ($addr, $rc);

   openOrDie(NEWADDRS, "> $MAILMGR_LIST_DIR/$name");
   openOrDie(ADDRS, "< $newaddrfile");
   while (<ADDRS>)
   {
      #  Add checks to prevent addition of malicious aliases
      print NEWADDRS $_;
   }
   close(ADDRS);
   close(NEWADDRS);
   
   # Make modifications to Mail List Manager aliases take effect
   $rc = runit(["/usr/bin/newaliases"], \$stdout, \$stderr);
   if ($rc != 0)
   {
     error(MSG_MAILMGR_ALIAS_ERROR);
     trace("Errors running newaliases: $stdout $stderr\n");
     return($rc);
   }
   return(0);
}

sub get_maillist_addrs
{
   my ($name) = @_;
   my (@addrs, $rc);

   openOrDie(ADDRS, "< $MAILMGR_LIST_DIR/$name");
   @addrs = ();
   foreach $addr (<ADDRS>)
   {
      chomp($addr);
      push(@addrs, $addr);
   }
   close(ADDRS);
   
   return(@addrs);
}

