#!/usr/bin/perl 
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# configureLilo    -  Configure /etc/lilo.conf required by ASM for kernel software upgrading.
#
# Configures /etc/lilo.conf

use ASM::Exec;
use ASM::Common;

my ($eSP_BIN, $rc, $stdout, $stderr, @opts, $configureLilo_usage, $boot_loc, $boot_part, $lilo_conf, $newlilo_conf);
my (@lines, @line);

$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_install.pl";

use strict;

@opts = ("part=s");
$configureLilo_usage = "\t\tor\nconfigureLilo.pl [--part=<boot partition>]";
commonInit(\@opts, $configureLilo_usage);

setFacility('syscfg');

use vars qw/ $opt_help $opt_part /;

$lilo_conf = "/etc/lilo.conf";
$newlilo_conf = "/etc/lilo.conf.new";
#$boot_loc = "/boot";
#  Use / instead of since /boot may be mounted from somewhere else.
#  In ASM /boot uses same partition as /.
$boot_loc = "/";

# Get command line options and check 'em out to see if they're okay.
&GetOptions(@opts);

if ($Getopt::Long::error){  # There was an error in parsing the arguments
  exit 2;
}

if ($opt_help)  {
  printUsage();
}

if ($opt_part)
{
   $boot_part = $opt_part;
   #  Validate it.
}
else
{
   # Determine partition of /boot
   $rc = runit(["/bin/df --portability $boot_loc"], \$stdout, \$stderr);
   if ($rc != 0)
   {
      trace("Error: problem locating $boot_loc partition.");
      trace("Output of df command: $stdout $stderr");
      exit(1);
   }

   #  Parse output of df command.  Expected Output Format:
   #         Filesystem         1024-blocks      Used Available Capacity Mounted on
   #         /dev/hda1                38859      2442     34411       7% /boot
   #
   $boot_part = "";
   @lines = split(/\n/, $stdout);
   if ($#lines > 0)
   {
      @line = split(/ /, $lines[1]);
      if ($#line > -1)
      {
         $boot_part = $line[0];
         trace("Found boot partition: $boot_part in df line output: $lines[1]");
      }
      else
      {
         # Unexpected output 
         trace("Error: problem locating $boot_loc partition.");
         trace("Unexpected Output of df command: $stdout $stderr");
         exit(1);
      }
   }
   else
   {
      # Unexpected output 
      trace("Error: problem locating $boot_loc partition.");
      trace("Unexpected Output of df command: $stdout $stderr");
      exit(1);
   }
}

#  Generate /etc/lilo.conf File.
openOrDie(LILO, "> $newlilo_conf");

print LILO "boot=$boot_part\n";
print LILO "map=/boot/map\n";
print LILO "install=/boot/boot.b\n";
print LILO "prompt\n";
print LILO "timeout=50\n";
print LILO "default=linux\n";
print LILO "\n";
print LILO "image=/boot/vmlinuz\n";
print LILO "	label=linux\n";
print LILO "    initrd=/boot/initrd\n" if ($boot_part =~ m@/dev/sd.+@); # SCSI drive
print LILO "	read-only\n";
print LILO "	root=$boot_part\n";

close LILO;

#  Need to verify the new lilo.conf file changes.  How??

$rc = runit(["/bin/cp -f $lilo_conf $lilo_conf.prev"], \$stdout, \$stderr);
if ($rc != 0)
{
   trace("Warning: Couldn't save old $lilo_conf file.");
   trace("Output of cp command: $stdout $stderr");
}

$rc = runit(["/bin/cp -f $newlilo_conf $lilo_conf"], \$stdout, \$stderr);
if ($rc != 0)
{
   trace("Error: Couldn't overwrite $lilo_conf file with ASM lilo changes.");
   trace("Output of cp command: $stdout $stderr");
   exit(1);
}
$rc = runit(["/bin/rm -f $newlilo_conf"], \$stdout, \$stderr);
if ($rc != 0)
{
   trace("Warning: Couldn't remove $newlilo_conf file.");
   trace("Output of rm command: $stdout $stderr");
}

trace("Successfully wrote ASM changes to $lilo_conf file.");

#  Run with the new lilo changes.
trace("Running command: /sbin/lilo");
$rc = runit(["/sbin/lilo"], \$stdout, \$stderr);
if ($rc != 0)
{
   trace("Error:  Problem running lilo with ASM lilo changes.");
   trace("Output of lilo command: $stdout $stderr");
   exit(1);
}
exit(0);
