#!/usr/bin/perl -Uw
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# configurePHP
#
# Pass '--enable' or '--disable'
# Will configure production web server to use (or not) PHP

use ASM::Exec;
use ASM::Common;
commonInit(['enable', 'disable'],'[--enable|--disable]');

use vars qw! $opt_enable $opt_disable !;

setFacility('sysuser');

if (!$opt_enable && !$opt_disable) {
  logUsage();
  exit(3);
}

my $config_file = "/opt/IBMHTTPServer/conf/httpd.conf";

openOrDie(IN, "<$config_file");
my @config = <IN>;
close(IN);

# Get rid of any PHP stuff already in there
remove_php(\@config);		# this is all we need for --disable

if ($opt_enable) {
  # Now, add module load
  add_after(\@config,
	    "^\\s*LoadModule",
	    "LoadModule php4_module libexec/libphp4.so\n");
  add_after(\@config,
	    "^\\s*AddModule",
	    "AddModule mod_php4.c\n");
  # And again for AddType
  add_after(\@config,
	    "^\\s*AddType",
	    "AddType application/x-httpd-php4 .php\n",
	    "AddType application/x-httpd-php .php\n");

}

# Save new file 
openOrDie(OUT, ">${config_file}-new");
printOrDie(OUT, @config);
closeOrDie(OUT);

# Backup old file
copyOrDie($config_file, "${config_file}.bak");

# Use copy over the original file, instead of rename,
# to preserve ownership and permissions.
copyOrDie("${config_file}-new", $config_file);

my $exitrc = 0;

# # Fix permissions on the "new" config file
# my ($stdout,$stderr);
# my $rc = runit(["chown", "ihsadmin:ihsadmin", $config_file], \$stdout, \$stderr);
# if ($rc) {
#   print STDERR $stderr;
#   $exitrc = $rc;
# }

# Restart the web server
$rc = runit(["/etc/rc.d/init.d/ibmhttpd", "graceful"], \$stdout, \$stderr);
if ($rc) {
  print STDERR $stderr;
  $exitrc = $rc;
}

# Exit non-zero if chown or restart failed,
# errors should be on stderr
exit($exitrc);

sub add_after
{
  my ($aref, $regex, @newlines) = @_;
  my $lastline = -1;
  my $i = 0;
  foreach my $line (@{$aref}) {
    if ($line =~ /$regex/) {
      $lastline = $i;
    }
    ++$i;
  }
  if ($lastline == -1) {
    $lastline = $i-1;		# put at end if not found
  }
  splice(@{$aref}, $lastline+1, 0, @newlines);
}

# Get rid of all references to PHP
sub remove_php {
  my ($aref) = @_;
  my @new;
  foreach my $line (@{$aref}) {
    push(@new, $line) unless $line =~ /php/i;
  }
  @{$aref} = @new;
}
