#!/usr/bin/perl -Uw
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# configureSendmail
#
# Reads sendmail config from csm.data
# Creates, installs, and activates a new /etc/sendmail.cf file
# using the sendmail m4 system.

use ASM::Exec;
use ASM::Common;
commonInit([],'');

setFacility('sysuser');

my $sendmail_cf = "/etc/sendmail.cf";
my $sendmail_mc = "/etc/sendmail.mc";

# Data from the csm.data file 
my $smtp_server;
my $postmaster_email = "/dev/null"; # default to throw away email

# Read csm.data file
openOrDie(DATA, "<$setup_data_file");
while (<DATA>) {
  chomp;
  if (/^smtp\.server=/) {
    $smtp_server = $';
  }
  if (/^postmaster\.email=/) {
    $postmaster_email = $';
  }
}
close(DATA);

# Build the M4 input file for the sendmail configuration
# The base configuration is based on RedHat's


  # Start with /etc/sendmail.mc
  # If no smarthost and we have one, then add it

my @config;
push(@config, "# Sendmail configuration created by $0.\n");

openOrDie(IN, "<$sendmail_mc");
while (<IN>) {
  if (/Sendmail configuration created by $0/io
      || /SMART_HOST/) {
    next;			# skip our own comment, and SMART_HOST
  }
  push(@config, $_);
}
close(IN);

if ($smtp_server) {
  push(@config, "define(`SMART_HOST', `$smtp_server')\n");
}

openOrDie(OUT, ">$sendmail_mc");
print OUT @config;
close(OUT);

# "Compile" it
my ($stdout,$stderr);
my $rc = runit(["/usr/bin/m4", "--fatal-warnings", $sendmail_mc], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_SENDMAIL_UPDATE_ERROR);
  trace("Errors from m4: $stderr\n");
  exit(1);
}

# Backup config if not previously done (at least for development
# so I don't have to completely re-install to fix myself...)
if (! -f "${sendmail_cf}-backup") {
  system("cp $sendmail_cf ${sendmail_cf}-backup");
}

# Replace config
openOrDie(OUT, ">$sendmail_cf");
print OUT $stdout;
close(OUT);

# Restart sendmail
$rc = runit(["/etc/rc.d/init.d/sendmail", "restart"], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_SENDMAIL_UPDATE_ERROR);
  trace("Errors restarting sendmail: $stdout $stderr\n");
}

# Fix Aliases
#
# We want to alias 'root' to $postmaster_email
# (We should already have all other users on the box
# aliased to root, but that's not managed here.)
# See man alias(5) for format of /etc/aliases.
# 
if ($postmaster_email) {
  $rc = runit(["/opt/CSM/scripts/setEmailAlias.pl", "--from=root",
	       "--to=$postmaster_email"], \$stdout, \$stderr);
} else {
  $rc = runit(["/opt/CSM/scripts/setEmailAlias.pl", "--from=root",
	       "--delete"], \$stdout, \$stderr);
}
if ($rc != 0) {
  error(MSG_SENDMAIL_UPDATE_ERROR);
  trace("Errors changing alias: $stdout $stderr\n");
}


audit(MSG_SENDMAIL_CONFIGURED, $smtp_server, $postmaster_email);
exit(0);
