#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
# configureWebservers.pl - Sets hostnames into web server config files
# Also make sure hostname is in /etc/hosts
#


use ASM::Common;
use ASM::Exec;
commonInit([],"");

use strict;
BEGIN { $^W = 1; }

use vars qw! $script_path !;

my @configFiles = ("/opt/CSM/WebServer/conf/httpd.conf",
		   "/opt/IBMHTTPServer/conf/httpd.conf",
		   "/opt/IBMHTTPServer/conf/admin.conf",
		  );

# Get hostname
chomp(my $hostname = `hostname`);

# Check for launchpad-broken name with no domain name
if ($hostname !~ /\./) {
  # No domain.  See if we can figure it out.  There might be a domain
  # name in /etc/sysconfig/network
  local *IN;
  my $domainname;
  if (open(IN, "</etc/sysconfig/network")) {
    while (<IN>) {
      chomp;
      if (/^DOMAINNAME=(.*)/) {
	$domainname = $1;
	last;
      }
    }
    close(IN);
  }
  if (defined($domainname)) {
    $hostname .= "." . $domainname;

    my ($rc,$stdout,$stderr);
    $rc = runit(["hostname", $hostname], \$stdout, \$stderr);

    local *OUT;
    if (open(OUT, ">/etc/HOSTNAME")) {
      print OUT "$hostname\n";
      close(OUT);
    }
  }
}

foreach my $file (@configFiles) {
  if (-f $file) {
    configure($file, $hostname);
  }
}
check_etc_hosts();

exit(0);

sub check_etc_hosts {
  # Is it in there?
  local *IN;
  my $found = 0;
  if (open(IN, "</etc/hosts")) {
    while (<IN>) {
      if (/$hostname/) {
	$found = 1;
	last;
      }
    }
    close(IN);
  }
  if (!$found) {
    local *OUT;
    if (open(OUT, ">>/etc/hosts")) {
      my @hostnames;
      trace("Adding hostname $hostname to /etc/hosts");
      push(@hostnames, $hostname);
      if ($hostname =~ /\./) {
	push(@hostnames, $`);	# add alias without hostname
      }
      print OUT get_ip_address() . " " . join(' ',@hostnames) . "\n";
      close(OUT);
    }
  }
}

sub get_ip_address {
  my @foo = `$script_path/device.pl --get --all`;
  foreach my $line (@foo) {
    my ($iface, $address, @rest) = split(' ',$line);
    if ($iface !~ /^lo\d*/) {
      return $address;
    }
  }
}

sub configure {
  my ($file, $hostname) = @_;

  audit(MSG_CONFIGURING_WEB_SERVER, $hostname, $file);

  my @lines;
  if (!open(IN, "<$file")) {
    error(MSG_COULD_NOT_OPEN_FILE, $file);
    exit(1);
  }
  my $found = 0;
  while (<IN>) {
    if (!$found && /^\s*ServerName/) {
      $found = 1;
      push(@lines, "ServerName $hostname\n");
    } elsif ($found && /^\s*ServerName/) {
      s/^/#/;
      push(@lines, $_);
    } else {
      push(@lines, $_);
    }
  }
  close(IN);
  if (!$found) {
    push(@lines, "ServerName $hostname\n");
  }
  openOrDie(OUT, ">$file");
  print OUT @lines;
  close(OUT);
}
