#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# consolechng.pl - allows the user to change the console
#
# Sarah McCain <smccain@us.ibm.com> (06 October 1999)
#

# For documentation on setting the Linux console,
# see <path to Linux kernel source>/Documentation/serial-console.txt.

use ASM::Common;
use ASM::Exec;
commonInit(["get", "setvideo", "setserial1", "setserial2"],
	   "--get | --setvideo | --setserial1 | --setserial2");

$lilo_file = "/etc/lilo.conf";
$run_lilo = "/sbin/lilo";
setFacility('syscfg');

if($opt_get) {
	$console = &getCurrConsole();
	print $console . "\n";
} elsif($opt_setserial1){
	&chngTheConsole("SERIAL1");	
} elsif($opt_setserial2){
	&chngTheConsole("SERIAL2");    
} elsif($opt_setvideo) {
	&chngTheConsole("VIDEO");	
} else {
  logUsage();
  exit(1);
}

exit(0);

#######################################
#	Subroutines BaBAAY!
#######################################

# Get the current console
sub getCurrConsole {
	my $current_console;
	my $k = 0;

	if (!open(LILO, "<$lilo_file")) {
	  error(MSG_COULD_NOT_OPEN_FILE, $lilo_file);
	  exit(4);
	}

        while(<LILO>) {

	# Find the append line for kernel params
		if (/^\s*append\s*=\s*"\s*(.*)\s*"\s*/) {
			@append_line = split(/\s+/, $1);

	# extract the kernel params with console		
			for (@append_line){
				if ( /console=/ ){
					push (@consoles, $append_line[$k]);
				} else {
					push (@params, $append_line[$k]);
				}
				
				$k++;
			
			} #for
			($key, $val) = split(/=/, $consoles[$#consoles], 2);

	# Check for the last console parameter passed, it is the main console
			if ($key =~ /^console/) {
				if($val =~ /^tty0/) {
					$current_console = "VIDEO";
					#print $current_console "in video \n";
				} elsif ($val =~ /^ttyS0/) {
					$current_console = "SERIAL1";
					#print $current_console "in serial1 \n";

				} else {
				$current_console = "SERIAL2";
				} #else
				
				last;
			} *if
		}#if

	} #while
	close(LILO);

	if (!$current_console) {
	  $current_console = "VIDEO";
	}

	return $current_console;
} #getCurrConsole


# Change main console to video, serial1, or serial2
sub chngTheConsole{

 	my $console_choice = @_[0];	
	my (@line);
	my $i = 0;
	my $n = 0;
	my $k = 0;

	my ($setserial) = 0;
	my ($setappend) = 0;

	if (!open(LILO, "<$lilo_file")) {
	  error(MSG_COULD_NOT_OPEN_FILE, $lilo_file);
	  exit(4);
	}
	
        while(<LILO>) {
	
	# make sure to set up all params correctly for a serial port config
	
		if((/^\s*serial/) && ($console_choice eq "SERIAL1")) {
		  # Set LILO to take input from serial
			$line[$i] = "serial=0,9600n8\n";
			  $setserial = 1;
		} elsif ((/^\s*serial/) && ($console_choice eq "SERIAL2")) {
		  # Set LILO to take input from serial
			$line[$i] = "serial=1,9600n8\n";

			  $setserial = 1;
	# pass the correct params for the main console to be as selected by the user
		} elsif(/^\s*append\s*=\s*"\s*(.*)\s*"\s*/) {
		  # Set Linux kernel's console
			$n = 0;
			@append_line = split(/\s+/, $1);

                	for (@append_line){
				if ( /console=/ ){
                                	push (@consoles, $append_line[$k]);
                                } else {

		# store any extra params passed to the kernel
                                        push (@params, $append_line[$k]);
                                } #else
                        	$k++;
                	} #for
	
			if($console_choice eq "VIDEO"){
			  # Put video console (tty0) last to make it the
			  # default, but include ttyS0 also.
				$line[$i] = "\tappend = \"console=ttyS0,9600 console=tty0";
			} elsif($console_choice eq "SERIAL1"){
			  # Move ttyS0 to end so it's the default
			  # leave tty0 also so output will go to screen
			  # if present.
				$line[$i] = "\tappend = \" console=tty0 console=ttyS0,9600";
			} else {
			  # Use ttyS1 at end so it's the default,
			  # leave tty0 also so output will go to screen
			  # if present.
				$line[$i] = "\tappend = \"console=tty0 console=ttyS1,9600";
			} #else

			#Add any other parameters passed to the kernel back on the append line
			
			$line[$i] .= " " . join(" ", @params);

		# store the newly modified append line for writing
			$line[$i] .= "\"\n";
	
			$setappend = 1;
		} else {	
		
		# store the old, unchanged lines of lilo.conf
			$line[$i] = $_;

		} #else

			$i++;
	} #while

	close(LILO);
	if (!$setserial && $console_choice =~ /^SERIAL/) {
	  # There was no 'serial' line in the lilo.conf
	  # Add one at the START if we want a serial console
	  if ($console_choice eq "SERIAL2") {
	    # Set LILO to take input from serial 2
	    unshift(@line,"serial=1,9600n8\n");
	  } else {
	    # Set LILO to take input from serial 1
	    unshift(@line,"serial=0,9600n8\n");
	  }
	}
	if (!$setappend && $console_choice =~ /^SERIAL/) {
	  # There was no 'append' line in the lilo.conf
	  # Add one at the START if we want a serial console
	  if ($console_choice eq "SERIAL2") {
	    # Set LILO to take input from serial
	    unshift(@line,"append = \"console=tty0 console=ttyS1,9600\"\n");
	  } elsif ($console_choice eq "SERIAL1"){
	    unshift(@line,"append = \"console=tty0 console=ttyS0,9600\"\n");
	  }
	}

	# Now write the updated information back to the lilo.conf file
	if (!open(LILO, ">$lilo_file"))
        {
           error(MSG_COULD_NOT_CREATE_FILE, $lilo_file);
           exit(4);
        }
	print LILO @line;
	close (LILO);

	# Remove /etc/ioctl.save so that 'init' won't mess up our
	# baud rate when we reboot.  Typically, 'init' will have saved
	# the 38400 rate of the Linux virtual terminal, and will then
	# try to set its 'console' to that rate when it starts.
	unlink("/etc/ioctl.save");

	#run lilo!!

	my ($rc,$stdout,$stderr);
	$rc = runit([$run_lilo], \$stdout, \$stderr);
	if ($rc) {
	  trace("Error running lilo: rc=$rc\n$stdout\n$stderr");
	  exit(5);
	}
	audit(MSG_UPDATED_SYSTEM_CONSOLE, $console_choice);
} #chngTheConsole

