#!/usr/bin/perl 
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# displayMaillist.pl
#
# Shows information about a Mail List.
#
# Usage:
#   displayMaillist.pl [--name=<list name>] [--help]
#   
#   Shows the currently configured Mail Lists along with the email addresses.
#   When a mail list is specified, the configuration info about that list is shown.

use ASM::Exec;
use ASM::Common;

my (@eSP_BIN, $rc, $stdout, $stderr, @opts, $displayMaillist_usage, $listname, @lists, $addrlist);
$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_mailmgr.pl";

#  Parse the arguments
@opts = ("name=s");
$displayMaillist_usage = "[--name=xxx]";

commonInit(\@opts, $displayMaillist_usage);

setFacility('sysuser');

use vars qw/ $opt_help $opt_name /;

# Get command line options and check 'em out to see if they're okay.

&GetOptions(@opts);

if ($Getopt::Long::error){  # There was an error in parsing the arguments
  exit 2;
}

if ($opt_help)  {
  printUsage();
}

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;

   print "$exitmsg\n";
   trace("$exitmsg");
   if ($exitrc != 0)
   {
      trace("Display Mail List failed. displayMaillist.pl exiting with rc=" . $exitrc);
      error(MSG_DISPLAY_MAILLIST_FAILED, $exitrc);
   }
   else
   {
      audit(MSG_DISPLAY_MAILLIST_COMPLETED, 0);
   }
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.  Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

audit(MSG_DISPLAY_MAILIST_STARTED);

if ($opt_name)
{
   audit(MSG_DISPLAY_MAILLIST, $opt_name);
   #  Retrieve the list's password
   $passwd = &get_maillist_passwd($opt_name);
   print "$passwd";
   $rc = &get_maillist_anypost($opt_name);
   if ($rc)
   {
      print " ANYPOST";
   }
   else
   {
      print " RESTRICTEDPOST";
   }
   $rc = &get_maillist_subscribe($opt_name);
   if ($rc)
   {
      print " OPEN";
   }
   else
   {
      print " APPROVAL";
   }
   $addrlist =  join(" ", &get_maillist_addrs($opt_name));
   print " $addrlist";
   print "\n";
}
else
{
   audit(MSG_DISPLAY_MAILLISTS);
   @lists = &get_maillists();
   foreach $listname (@lists)
   {
      chomp($listname);
      print $listname;
      $addrlist =  join(" ", &get_maillist_addrs($listname));
      print " $addrlist";
      print "\n";
   }
}

&cleanup_exit("", 0);
