#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


# edQuota.pl creates/modifies the quota of a linux user.
#
# Pierre Carlson <mpcarl@us.ibm.com> (12 October 1999)
#
# This script is called by the /usr/sbin/edquota program by setting the EDITOR
# environment variable before calling /usr/sbin/edquota.  It will takes the name
# of the file given on the command line and modifies it with the new quota data.
# The hardlimit is received from the environment variable USER_HARD_LIMIT

use ASM::Common;
commonInit([],"");

$fileName = $ARGV[0];
$hardLimit = $ENV{"USER_HARD_LIMIT"};
$device    = $ENV{"USER_QUOTA_DEVICE"};

# open the file
openOrDie(FH, "+< $fileName");
# Assign to an array
@ARRAY = <FH>;
# Modify the second line of the array
$i = 0;
foreach $line (@ARRAY) {
    #print STDOUT "checking $line \tfor $device\n";
    if ($line =~ m|^$device|) {
	#print STDOUT "Match on ".$i."\n";
	# this is the line
	$ARRAY[$i] =~ s/soft = [0-9]+/soft = $hardLimit/;
	$ARRAY[$i] =~ s/hard = [0-9]+/hard = $hardLimit/;
	last;
    }
    #print STDOUT "NO match on ".$i."\n";
    $i++;
}

# write back the array and close file
seek(FH,0,0)              or do {
  trace("error seeking in $fileName: $!");
  error(MSG_ERROR_WRITING_FILE, $fileName);
  exit(3);
};
print FH @ARRAY           or do {
  trace "Printing: $!";
  error(MSG_ERROR_WRITING_FILE, $fileName);
  exit(3);
};
truncate(FH,tell(FH))     or do {
  trace "Truncating: $!";
  error(MSG_ERROR_WRITING_FILE, $fileName);
  exit(3);
};
close(FH)                 or do {
  trace "Closing: $!";
  error(MSG_ERROR_WRITING_FILE, $fileName);
  exit(3);
};

exit 0;
