#!/usr/bin/perl
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# extract  <url> <file>
#
# Sends an HTTP 1.0 request to fetch the page at the host location
# indicated by the specified <url>.  The default protocol is 'http'.
# The default host is 'localhost'.  The default port is 80.
# The results (error messages or the specified page source) are
# sent to <file> (if specified) or STDOUT.
#
# This script was shamelessly plagerized (with permission) from an
# example at http://step.sdsc.edu/s96/perl/sockets.
#
# Jeff Cates, <jcates@us.ibm.com> (20 October, 1999)
#
#

use Socket;

use strict;

use ASM::Common;

commonInit([],"");

setFacility('initsys');

sub extract
{     	# begin subroutine
   # Get input arguments and initialize variables.
   my ($input, $outfile) = @_;
   my ($error, $port, $hostname, $xf, $oldout, $protocol, $url, $remote_host);
   my ($sockaddr, $name, $aliases, $proto, $type, $len, $thisaddr, $this, $that, $url_request);
   my ($thataddr, $status, @output, $query_string);
   my ($savehand);
   
   $port = '';
   chomp($hostname=`hostname`);
   $error = 0;
   
   # If outfile specified, open it.
   if ($outfile && (open(MYHAND, "> $outfile"))) {
     $savehand = *MYHAND;
     $xf = 1;
   }
   else {
     $savehand = *STDOUT;
     $xf = 2;
   }
   $oldout = select($savehand);
   
   # Next, split off the protocol. If it isn't http, change it to http
   if ($input =~ /:\/\//){
     ($protocol, $url) = split(/:\/\//, $input);
   } else {
     $protocol = 'http';
     $url = $input;
   }
   
   # The very first component of the URL will be the remote host name.
   # Only one split with respect to '/' is desired.
   ($remote_host, $url) = split(/\//, $url, 2);
   
   # get the port, if one is specified
   if ($remote_host =~ /:/){
     ($remote_host, $port) = split(/:/,$remote_host);
   }
   
   # if there is no hostname, use the locahost
   unless ($remote_host){ $remote_host = $hostname; }
   
   # The format conversion string pattern for socket addresses
   $sockaddr = 'S n a4 x8';
   
   # Get the protocol number for tcp
   ($name, $aliases, $proto) = getprotobyname('tcp');
   
   # If the port isn't specified, try to get it's official value
   unless ($port){
     ($name, $aliases, $port) = getservbyname($protocol, 'tcp');
   }
   
   # not all systems know what the port number of http is, so
   # if it doesn't, we hardwire the value.
   unless ($port){ $port = 80;}
   
   # now get the local and remote IP addresses
   ($name, $aliases, $type, $len, $thisaddr) = gethostbyname($hostname);
   ($name, $aliases, $type, $len, $thataddr) = gethostbyname($remote_host);
   
   # now convert them to machine-usable format.
   $this = pack($sockaddr, &AF_INET, 0, $thisaddr);
   $that = pack($sockaddr, &AF_INET, $port, $thataddr);
   
   # Establish socket.
   if (socket(S, &AF_INET, &SOCK_STREAM, $proto)) {
     # Now bind it.
     if (bind(S, $this)) {
       # Now make connection.
       if (connect(S, $that)) {
   
         # Set socket to be command buffered
         select(S); $| = 1; select($savehand);
         $url_request = "/$url";
         # Now we undo the split into URL and QUERY_STRING to make the call
         # We didn't need to split them up, but it is useful to do so for debugging
         if ($query_string){ $url_request .= "?$query_string";}
         # Send the request to the remote system. Very simple header.
         print S "GET $url_request HTTP 1.0\n\n";
   
         # Get the status line since this is not a non-parse header script..
         $status = <S>;
   
         # Handle results
         if ($status =~ '200'){
           while (<S>){ print;}
         }
         else {
           @output = <S>;
           print @output;
         }
   
       } # end connect OK
       else {
         error(MSG_UNABLE_TO_CONNECT, "$remote_host:$port");
         $error = 3;
       }
     } # end bind OK
     else {
       error(MSG_UNABLE_TO_BIND_SOCKET);
       $error = 2;
     }
   } # end socket OK
   else {
     error(MSG_UNABLE_TO_OPEN_SOCKET);
     $error = 1;
   }
   
   # Restore environment
   if ($xf) { close($savehand); }
   select($oldout);

   return($error);

} # end extract function

1;
