#!/usr/bin/perl -w
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2001. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# nfs.pl implements filesystem functions for ASM.
# Bernd Horn <bhorn@us.ibm.com> (April 23, 2001)

use ASM::Exec;
use ASM::Common;

commonInit( [ "list:s",
	      "status",
	      "debug" ],
	    "--list [all] --status");

use vars qw! $opt_list $opt_status $opt_debug !;

#
#
#
#

##############################################################################
#
# Check parameters
#
#if (! ($opt_list || $opt_add || $opt_delete || $opt_activate || $opt_deactivate)) {
#  logUsage();
#  exit(1);
#}

##############################################################################
#
# Define variables
#
my $DEBUG = 0;
$DEBUG = 1 if $opt_debug;

##############################################################################
#
# 
#
if (defined($opt_list))
{
    my (%active_filesystems);

    local(*FILE);
    openOrDie(FILE, "/proc/mounts");
    while (<FILE>)
    {
	my ($fs_spec, $fs_file, $remaining) = split;
	$active_filesystems{$fs_file} = $fs_spec if ($fs_file =~ m@^/home@ || $fs_file =~ m@^/net/home.*@);
    }
    closeOrDie(FILE);

    foreach $fs (getFilesystemTable("/etc/fstab"))
    {
	my $status = exists($active_filesystems{${$fs}[1]})? "1" : "0";
        next if ($opt_list ne "all" &&
		 $status == 0);
        print "$status " if ($opt_status);
	print ${$fs}[1]." ".${$fs}[0]."\n";
    }
}

exit(0);


#
# Subroutine: getFilesystemTable
#
# Returns an array representation of filesytem tables
#
sub getFilesystemTable($) {
    my ($table) = @_;

    my @file_systems;

    open FSTAB, "< $table";
    while (<FSTAB>)
    {
	next if (/^\#/ || /^\s*$/);
	($fs_spec, $fs_file, $fs_vfstype, $fs_mntops, $fs_freq, $fs_passno) = split;
	next if !defined($fs_spec);
	if (($fs_file =~ m@^/net@) ||
	    ($fs_file =~ m@^/home@))
	{
	    push @file_systems, [$fs_spec, $fs_file, $fs_vfstype, $fs_mntops, $fs_freq, $fs_passno];
	}
    }
    close FSTAB;

    return @file_systems;
}








