#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# ftpd.pl - turns ftp on and off  for a given machine
#
# Sarah McCain <smccain@us.ibm.com> (06 October 1999)
#

use strict;
my (@ftpd_opts, $ftpd_usage, $return_ftpstat, @file);

@ftpd_opts = ("disable", "enable", "status");
$ftpd_usage = "ftpd.pl --disable\nftpd.pl --enable\nftpd.pl --status\n";
use ASM::Common;
commonInit(\@ftpd_opts, $ftpd_usage);
setFacility('syscfg');

use vars qw/ $opt_enable $opt_disable $opt_status $inetd_file $run_inet /;

use Fcntl;
use DB_File;

if ($opt_enable) {
	&turnFtpOnOff("enable");
	&reReadInetd();
} elsif($opt_disable) {
	&turnFtpOnOff("disable");
	&reReadInetd();
} elsif($opt_status) {
	$return_ftpstat = &getFtpOnOff();
	print $return_ftpstat . "\n";
} else {
	&printUsage();
}#if---else

exit (0);


sub getFtpOnOff {

	my $ftp_state = "OFF";
	openOrDie(INETDCONF, "<$inetd_file");

	while (<INETDCONF>) {

		if (/^ftp\s+/) 
		{
		    $ftp_state = "ON";
		    last;
		} #if


	} #while

	close(INETDCONF);

	return $ftp_state;

} #getFtpOnOff


sub turnFtpOnOff {

	my $option = shift(@_);

	# open /etc/inetd.conf for read/write to edit ftp line

	if (!tie (@file, 'DB_File', "$inetd_file", O_RDWR, 0644, $DB_RECNO))
        {
           error (MSG_COULD_NOT_UPDATE_FILE, $inetd_file);
           exit(5);
        }
	my $i = 0;
        
	my @lineNumbers;
	while (defined $file[$i]) {

        	$_ = $file[$i];

		#find the WU ftp lines (TurboLinux might have more than one)
		if (/^#*ftp\s+.*in\.ftpd\s+/){
		    push(@lineNumbers, $i);
        	}
        	$i++;

	} #while


	if ($option eq "enable") {
	  # Enable the first line we found
	  # Make sure any others are disabled
	  $i = $lineNumbers[0];
	  $file[$i] =~ s/^#+//;
	  shift(@lineNumbers);
	  foreach $i (@lineNumbers) {
	    if ($file[$i] !~ /^#/) {
		$file[$i] =~ s/^/#/;
	      }
	  }
	  audit(MSG_ENABLED_FTP);
	} elsif ($option eq "disable") {
	  foreach $i (@lineNumbers) {
	    if ($file[$i] !~ /^#/) {
		$file[$i] =~ s/^/#/;
	      }
	  }
	  audit(MSG_DISABLED_FTP);
	} else {
	        untie @file;
		return (3);
	}


	untie @file;

} #turnFtpOnOff

# reload the inetd.conf file with changes
sub reReadInetd {
	
  my ($rc,$stdout,$stderr);
  $rc = runit([$run_inet], \$stdout, \$stderr);
  if ($rc) {
    trace("error reloading inetd: rc=$rc\n$stdout\$stderr");
    error(MSG_ERROR_RELOADING_INETD);
    exit(6);
  }


} #reReadInetd

