#!/usr/bin/perl 
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

use Getopt::Std;
use POSIX 'setsid';
use File::Copy;

use strict;
BEGIN { $^W = 1; }

use ASM::Common;
commonInit(['daemon','stop'],'grabber.pl [--daemon]');

setFacility('sysadmin');

# usage
# iterate forever used for generation of history logs.
# ./grabber.pl -d

# iterate only once, used for getting realtime statistics. 
# ./grabber.pl

my ($rc, $stdout, $stderr);
my $DEBUG = 1;
my $pid_file = "/opt/CSM/data/grabber.pid";
my $name = "grabber";		# name to look for in ps

use vars qw! $opt_daemon $opt_stop !;

if ($opt_stop) {
  signal_stop();
  exit(0);
}

if ($opt_daemon) {
  if (alreadyRunning($pid_file,$name)) {
    print STDERR "$0 already running\n";
    exit(0);
  } else {
    $SIG{TERM} = \&cleanup_and_exit;
    daemonize($pid_file);
  }
}

#################################
#DIRECTORY & FILES STRUCTURE
#################################
my($basedir)            = "/opt/CSM";
my($datadir)            = "/data";
my($statusdir)          = "/status";

while(1) {

   #-----------------------------------------------------------------------------------
   # A. Load the config file for grabber.pl.
   # OPEN UP THE CONFIG FILE TO DETERMINE HOW LONG TO SLEEP.  THE CONFIGURATION FILE IS
   # OBTAINED FROM USER CONFIGURATION AND STORED IN SETUP.DAT.  THE FIRST LINE OF SETUP.DAT
   # CONTAINS THE SLEEP TIME FOR THE PROGRAM TO SLEEP.  FOR MORE REFERENCES YOU MAY VIEW
   # SETUP.HTML - THE FILE THAT PRESENTS THE INTERFACE TO ALLOW THE USER TO CONFIGURE POLL TIME.
   # AND VIEW HealthServlet WHERE THE command=550 or command=551 or command=552.
   # 
   # ALSO RETRIVE HOW MANY FILES THE PROGRAM SHOULD BE STORING...
   #-----------------------------------------------------------------------------------
      
   my($q)=0;
   my(@configArray);
   openOrDie(CONFIGFILE, "< $basedir$datadir/setup.dat");
   while(<CONFIGFILE>) {
      $configArray[$q]=$_;
      $q++;
   }
   close(CONFIGFILE);

   #THE SECOND ITEM IS THE SNAPSHOTS POLLING RATE (ITS BACKWARDS FROM HOW ITS LISTED IN THE GUI)
   #-----------------------------------------------------------------------------------

   #All the list of files that are used by the system monitoring material.
   #THE FOLLOWING LIST ARE KEYWORDS OF WHAT TO SCAN FOR WITHIN THE SNAPSHOTS FILES.

   my(@procfiles);	
   @procfiles = ("/proc/net/dev",
		"/proc/loadavg",
		"/proc/meminfo",
		"/proc/uptime",
		"/proc/stat",
		"/proc/version",
                "ps",
                "df");

   #Open each file and print them out..

   my($timestamp);
   my($i)=0;

   # Always create a history file.
   # EACH FILE IS NAMED AFTER THE TIMESTAMP (REPLACE SPACE WITH .)
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
   # Try to format the timestamp fairly culturally neutral...
   $timestamp = sprintf("%04d-%02d-%02d.%02d:%02d:%02d",
			$year + 1900, $mon+1, $mday,
			$hour, $min, $sec);

   #$timestamp =~ s/ /\./gi;

   openOrDie(TARGET, "> $basedir$datadir$statusdir/$timestamp");
   $i=0;
   while(defined $procfiles[$i]){

      # ADD ON A DELIMETER SO THAT WAY WE KNOW WHERE THE PROC FILE BEGINS

      print TARGET "%%%$procfiles[$i]\n";
      print TARGET "$timestamp\n";

      # IF ITS NOT THE SPECIAL CASE OF THE PROGRAM DF, THEN OPEN UP THE PROC FILE
      # AND WRITE TO THE LOGS.  

      my $compare=$procfiles[$i];
      if($compare eq "df" || $compare eq "ps") {

        if($compare eq "df") {
           #EXECUTE THE PROGRAM DF TO GET A READING ON THE DRIVES.
           #USE BACKTICS TO PUT ALL OF DF TO THE VARIABLE $output.
	  # Note: GNU df wraps long lines unless you use --portability,
	  # which screws up our parsing of the output.
           my $output = `df --portability`;
           print TARGET $output;
        }
        if($compare eq "ps") {
	   my $output = `ps -auxwww`;
           print TARGET $output;
        }
      } else {

         #OPEN UP THE PROC FILES AND SAVE THEM.

         openOrDie(SOURCE, "< $procfiles[$i]");
         while (<SOURCE>) {
            print TARGET $_;
         }

         # CLOSE THE PROC SOURCE FILE, MOVE ON TO THE NEXT.  

         close(SOURCE); 
      }

      # PRINT THE END DELIMITER.
      print TARGET "%%%$procfiles[$i]\n"; 
       
      $i++;  
   }
   close(TARGET);

   # Create "now" file for refresh web stats
   if(! $opt_daemon)
   {
     copy("$basedir$datadir$statusdir/$timestamp",
	  "$basedir$datadir$statusdir/now");
   }
   #close(TARGETGRAPH);

   # THE ITERATION IS BASED UPON IF THE USER SETS THE -d OPTION
   # ITERATE FOREVER IF 	$opt_daemon
   # ONE INDIVIDUAL RUN IF 	$opt_daemon not defined
   # THIS IS BASED UPON THE COMMAND LINE OPTION.  
   #
   # EXAMPLES: 
   # A.  IF THE USER TYPES:
   #
   # "grabber.pl "
   #    THE PROGRAM WILL RUN ONLY ONCE.  THIS IS MOSTLY USED FOR 
   #    OBTAINING INSTANT STATUS.
   #
   # B.  IN CONTRAST, IF THE USER TYPES: 
   #
   # "grabber.pl -d" 
   #    THIS LOOPS THE PROGRAM INDEFINATELY.  THIS OPTION IS USED ON 
   #    SYSTEM STARTUP TO RECORD ALL LOGS.

   if($opt_daemon){
      sleep($configArray[2]);
   }

   #----------------------------------------------------------------------------------------------
   # NOW BASED UPON HOW MANY USER MAINTAINANCE SNAPSHOTS THE USER WISHES TO HOLD, SORT THE 
   # FILES WITHIN THE LOGS DIRECTORY, AND IF THE AMOUNT OF FILES IS GREATER THAN WHAT THE USER
   # HAS SPECIFIED, DELETE THE OLDEST FILES FIRST.   CONSEQUENTLY, THOSE FILES THAT HAVE BEEN
   # RECENTLY COPIED OR CREATED WILL BE SORTED AT THE TOP OF THE LIST AND FURTHER FROM THE DELETION
   # PROCESS.  THOSE FILES THAT HAVE EXISTED LONGEST WILL BE FIRST ON THE QUEUE TO BE DELETED. 
   #----------------------------------------------------------------------------------------------
   purgefiles($configArray[1]);

   # Exit loop unless we're supposed to loop forever
   last unless $opt_daemon;
} 

####################################################################################
####################################################################################
########################         End Program              ##########################
####################################################################################
####################################################################################

#----------------------------------------------------------------------------------------------
# NOW BASED UPON HOW MANY USER MAINTAINANCE SNAPSHOTS THE USER WISHES TO HOLD, SORT THE 
# FILES WITHIN THE LOGS DIRECTORY, AND IF THE AMOUNT OF FILES IS GREATER THAN WHAT THE USER
# HAS SPECIFIED, DELETE THE OLDEST FILES FIRST.   CONSEQUENTLY, THOSE FILES THAT HAVE BEEN
# RECENTLY COPIED OR CREATED WILL BE SORTED AT THE TOP OF THE LIST AND FURTHER FROM THE DELETION
# PROCESS.  THOSE FILES THAT HAVE EXISTED LONGEST WILL BE FIRST ON THE QUEUE TO BE DELETED. 
#----------------------------------------------------------------------------------------------
sub purgefiles {
   my($purgeNum) = @_;

   my(@filenames);
   my($k)=0;
   my($timestamp);

   #NOTE THIS ONLY PICKS UP THINGS WITH AN EXTENSION.  EG "NOW" IS NOT PICKED UP.
   while (<$basedir$datadir$statusdir/*.*>) {
      $filenames[$k]=$_;
      $k++;
   }
   
   # SORT EACH FILE IN ACCORDANCE TO ITS MODIFICATION DATE.
   # THE SORT FUNCTION TAKES AN OPTIONAL CODE BLOCK WHICH LETS YOU SPECIFY
   # A COMPARE ROUTINE OF YOUR OWN.  THE COMPARISON FUNC IS CALLED EACH
   # TIME SORT HAS TO COMPARE TWO VALUES.  <=> IS FOR SORTING NUMBERS:
   
   # FILECOUNTER IS THE CURRENT FILE SORTED IN ORDER FROM NEWEST TO OLDEST.
   my($fileCounter)=0;

   foreach my $fileN (sort {(stat($b))[9] <=> (stat($a))[9]} @filenames) {
      ($timestamp) = (stat($fileN))[9];
   #print "filecounter $fileCounter purgeNum:$purgeNum Purge Files set at : $purgeNum\n";
      if($fileCounter >= $purgeNum-1) {
         #print "deleting: $fileN\n";
         unlink $fileN;
      }
      $fileCounter++;
   }
} 

sub send_signal($) {
  my ($signal) = @_;
  # Send a signal to a running daemon to reload its variables or quit
  local(*IN);
  if (open(IN, "<$pid_file")) {
    chomp(my $pid = <IN>);
    close(IN);
    kill $signal, $pid;
  }
}

sub signal_stop {
  send_signal(15);
}

sub cleanup_and_exit {
  if ($opt_daemon) {
    unlink($pid_file);
  }
  exit(0);
}
