#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# httpaction.pl - Run actions against the apachectl and adminctl.
#
# Lewis Boehm <lboehm@us.ibm.com>
#
#

use ASM::Common;
use ASM::Exec;
use ASM::Log;

setFacility('initappl');

commonInit([], "");

# Get the server
$server = $ARGV[0] if ( ($ARGV[0] eq "adminctl") || ($ARGV[0] eq "apachectl"));
if ($server eq "") {
  error(MSG_USAGE_ERROR, $0, "$0 adminctl|apachectl [start|restart|stop|graceful]");
  exit(1);
}

# Get the Action
$action = $ARGV[1] if ( ($ARGV[1] eq "start") || ($ARGV[1] eq "restart") || ($ARGV[1] eq "stop") || ($ARGV[1] eq "graceful"));

my ($rc,$stdout,$stderr);

if ($action ne "")
{
   $cmd = "/opt/IBMHTTPServer/bin/" . $server . " " . $action;
   #die "Action failed: $cmd: $!" if system($cmd);
   $rc = runit([$cmd], \$stdout, \$stderr);
   if ($rc) {
     trace("$cmd failed: rc=$rc\n$stdout\n$stderr");
     exit(1);
   }
   $exitrc=0;
}
else
{
    if ($server eq "adminctl") 
    {
       $filename = "/opt/IBMHTTPServer/logs/admin.pid"
    }
    else
    {
       $filename = "/opt/IBMHTTPServer/logs/httpd.pid"
    }
    if (-e $filename)
    {
	$exitrc=0;
        #  Validate that the httpd in the pid file is actually running
        #  Sometimes the IHS start or restart fails without removing the pid file.
        #$rc = system("ps -p `cat $filename`> /dev/null 2>&1");
        #$rc >>= 8;
	$rc = runit(["ps -p `cat $filename`"], \$stdout, \$stderr);
        if ($rc != 0)
        {
	   $exitrc=1;
        }
    }
    else 
    {
	$exitrc=1;
    }
    
}

if ($action ne "")
{
  if ($action eq 'start') {
    audit(MSG_WEBSERVER_STARTED);
  } elsif ($action eq 'stop') {
    audit(MSG_WEBSERVER_STOPPED);
  } elsif (($action eq 'restart') || ($action eq 'graceful')) {
    audit(MSG_WEBSERVER_RESTARTED);
  }
}

exit($exitrc);
