#!/usr/bin/perl -w
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


# listLinuxUsers.pl lists linux users in CSMuser group
#
################################
# Let's use our needed modules #
################################
use ASM::Exec;
use ASM::Common;

use User::pwent;

commonInit( [ "user=s","help"],
	    "[--user=userid]");

use strict 'vars';
use vars qw! $opt_user  $opt_help!;
use vars qw! $CSMgrp!;

#############################################
# First we have our configuration variables #
#############################################
if($opt_help)
{
  logUsage();
  exit(0);
}
else
{
# Local Variables

    my %disabled = ();
    
# Make an associative array of users whose account is disabled

    openOrDie(PASSWDFILE,"/etc/shadow");
    while (<PASSWDFILE>) 
    {
	my ($userid,$userpass) = split /:/;                      # get password
	if ((substr $userpass, 0, 1) eq '!')                     # found the password 
	{  
	    $disabled{$userid} = 1;
        }
    }
    close(PASSWDFILE);
    
# get the group ID for CSMgrp
    my ($gname,$gpasswd,$gid,$remainder) = getgrnam($CSMgrp);

# Option user= specified (ie.  looking for a particular user);

    if (defined($opt_user))
    {
	my $pw = getpwnam($opt_user);
	if (!$pw)
	{
	    error(MSG_NO_SUCH_USER, $opt_user);
	    exit(4);
	}
	if (exists($disabled{$pw->name}))
	{
	    print "[".$pw->name."]:".$pw->uid.":".$pw->dir."\n";
	}
	else 
	{
	    print $pw->name.":".$pw->uid.":".$pw->dir."\n";
	}
    }   
    else
    {
# Looking for all defined users
	my @regularUsers = ();
	my @disabledUsers = ();

# find all users in passwd that are in $gid
	while (my $pw = getpwent)
	{
	    if ($pw->gid == $gid)                          # Check to see if the id is in CSMuser group
	    {
		if (exists($disabled{$pw->name}))
		{
		    push @disabledUsers , "[".$pw->name."]:".$pw->uid.":".$pw->dir."\n";
		}
		else 
		{
		    push @regularUsers,  $pw->name.":".$pw->uid.":".$pw->dir."\n";
		}
	    }
	}

	print (sort @regularUsers);
	print (sort @disabledUsers);
    }

    exit 0;
}

