#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# ----------------------------------------------------------------------
# Perl script to allow command line access to the SSL Key Management 
# functions
# 
# Pierre Carlson 10/28/1999
# ----------------------------------------------------------------------

# Modules to use
use Getopt::Long;
#require "/opt/CSM/scripts/csm_common.pl" ;
# our java class
$SSLCertMgr = "com.ibm.esp.csm.ssl.SSLCertMgr";
# other valid user id
#$ws_user_id = 99;
#$ws_user_id = 500; # FIX

use ASM::Common;
use ASM::Exec;

# Check the Args

$createUsageKDB  = "manageSSLCert.pl --create --KDB --keyFile <Key Database File> \n".
    "\t--password <password> [ -expiration <expiration in days> ] \n\n";
$createUsageReq  = "manageSSLCert.pl --create --Req --keyFile <Key File Name> \n".
    "\t--password <password> --certFile <file to store request> --label <Label> --version < X509V3 | X509V2 | X509V1 > \n".
    "\t--size [ 512 | 1024 ] --commonName <machine common name> --org <organization>\n".
    "\t--country <Country Code> --valid <Validity Period (days)> [ --orgUnit <Organization Unit>\n".
    "\t--locality <Locality> --state <State/Province> --zip <ZipCode>] \n\n";
$createUsageSelf = "manageSSLCert.pl --create --SelfCert --keyFile <Key File Name> \n".
    "\t--password <password> --certFile <file to store request> --label <Label> --version < X509V3 | X509V2 | X509V1 >\n".
    "\t--size [ 512 | 1024 ] --commonName <machine common name> --org <organization>\n".
    "\t--country <Country Code> --valid <Validity Period (days)> [ --orgUnit <Organization Unit> \n".
    "\t--locality <Locality> --state <State/Province> --zip <ZipCode>] \n\n";
$createUsage     = $createUsageKDB . $createUsageReq . $createUsageSelf;


$addUsageCert = "manageSSLCert.pl --add --Cert --keyFile <Key Database File> \n".
    "\t--password <password> --certFile <certificate file> --dataType < [ ASCII | BinDER > \n\n";
$addUsageCA   = "manageSSLCert.pl --add --CACert --keyFile <Key Database File> \n".
    "\t--password <password> --label <label> --certFile <certificate file> --dataType < [ ASCII | BinDER > \n\n";
$addUsage     = $addUsageCert . $addUsageCA;


$delUsageCert = "manageSSLCert.pl --del --Cert --keyFile <Key Database File> \n".
    "\t--password <password> --label <Label> \n\n";
$delUsageReq   = "manageSSLCert.pl --del --Req --keyFile <Key Database File> \n".
    "\t--password <password> --label <Label> \n\n";
$delUsageCA   = "manageSSLCert.pl --del --CACert --keyFile <Key Database File> \n".
    "\t--password <password> --label <Label> \n\n";
$delUsage     = $delUsageCert . $delUsageReq . $delUsageCA;

$listUsageCert = "manageSSLCert.pl --list   --Cert --keyFile <Key Database File> --password <password> \n\n";
$listUsageCA   = "manageSSLCert.pl --list   --CACert --keyFile <Key Database File> --password <password> \n\n";
$listUsageReq  = "manageSSLCert.pl --list   --Reqs --keyFile <Key Database File> --password <password> \n\n";
$listUsage = $listUsageCert . $listUsageCA . $listUsageReq;

$importUsage   = "manageSSLCert.pl --import --PKCS --keyFile <Key Database File> --password <password> \n".
    "--certFile <certFile> --certPassword <certificate password>\n\n";

$manageSSLCert_usage = $createUsage.$addUsage.$delUsage.$listUsage.$importUsage;

#print "args:\n\t" . join(":", @ARGV) . "\n";
#exit;
# parse the command line options and put them into the correct variables
@manageSSLCert_opts = ("create","add","del","list", "KDB", "Req", "SelfCert", "password=s", "Cert", "CACert", "Reqs", "keyFile=s", "password=s", "expiration=i", "label=s", "size=i", "commonName=s", "org=s", "country=s", "orgUnit=s", "locality=s", "state=s", "zip=s", "keyFile=s", "certFile=s", "version=s", "valid=i", "dataType=s", "certPassword=s", "PKCS", "import", "noRun",  "debug", "help"); 
#require '/opt/CSM/scripts/csm_common.pl';

commonInit(\@manageSSLCert_opts,
	   $manageSSLCert_usage);

if(defined $opt_help) {
    $opt_help = 1; # remove warning message
    logUsage();
    exit -1;
}

# test the options and see if they are correct for each type of work
if (defined $opt_create && !(defined $opt_add || defined $opt_del || defined $opt_list || defined $opt_import)) {
    &doCreate();
}
elsif (defined $opt_add && !(defined $opt_create || defined $opt_del || defined $opt_list || defined $opt_import)) {
    if (defined $opt_keyFile && -e $opt_keyFile) {
	&doAdd();
    } else {
        error(MSG_FILE_DOES_NOT_EXIST, $opt_keyFile);
	exit 3;
    }
}
elsif (defined $opt_del && !(defined $opt_add || defined $opt_create || defined $opt_list || defined $opt_import)) {
    if (defined $opt_keyFile && -e $opt_keyFile) {
	&doDel(); 	
    } else {
        error(MSG_FILE_DOES_NOT_EXIST, $opt_keyFile);
	exit 3;
    }
}
elsif (defined $opt_list && !(defined $opt_add || defined $opt_del || defined $opt_create || defined $opt_import)) {
    if (defined $opt_keyFile && -e $opt_keyFile) {
	&doList();
    } else {
        error(MSG_FILE_DOES_NOT_EXIST, $opt_keyFile);
	exit 3;
    }    
}
elsif (defined $opt_import && !(defined $opt_create && defined $opt_add || defined $opt_del || defined $opt_list)) {
    &doImport();
}

else {
    &errOpt($usage);
}

exit 0;


sub doCreate {
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# Function for creating key database
    if (defined $opt_KDB) {
	$opt_KDB = 1; # removes the warning
	if (defined $opt_keyFile && defined $opt_password) {
	    $args = "cdb " .  $opt_keyFile . " " . $opt_password;
	    if (defined $opt_expiration) {
		$args = $args . " " . $opt_expiration;
	    }
	    &runJava($args);
	} else {
	    &errOpt($createUsageKDB);
	}	
    }
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# Function for creating SSL Certificate request or Self Signed Certificate
    elsif (defined $opt_Req || $opt_SelfCert) {	
	if (defined $opt_Req) { # Request
	    $args = "ccr";
	}
	else { # Self Signed
	    $opt_SelfCert = 1;
	    $args = "css";
	}
	# required options
	if (defined $opt_keyFile && defined $opt_password && defined $opt_certFile && defined $opt_label && 
	    defined $opt_version && defined $opt_size && defined $opt_commonName && defined $opt_org && 
	    defined $opt_country && defined $opt_valid ) { # OK
	    if (defined $opt_valid && defined $opt_version) {
		if ((($opt_valid == 512) || ($opt_valid == 1024)) &&
		    ($opt_version eq "X509V1") ||  ($opt_version eq "X509V2") || ($opt_version eq "X509V3")) {	
		    $args = $args . " " . $opt_keyFile . " " . $opt_password . " " . $opt_certFile . " \"" .$opt_label . "\" " . 
			$opt_version . " " . $opt_size . " " . $opt_commonName . " " . $opt_org . " " . $opt_country . " " . $opt_valid;
		    if (defined $opt_orgUnit) {
			$args = $args . " -ou " . $opt_orgUnit; }
		    if (defined $opt_locality) {
			$args = $args . " -lo " . $opt_locality; }
		    if (defined $opt_state) {
			$args = $args . " -st ". $opt_state; }
		    if (defined $opt_zip) {
			$args = $args . " -zp " . $opt_zip; }
		    
		    &runJava($args);
		    return;
		}
	    }
	}
	&errOpt($createUsageReq . $createUsageSelf);
    }
    else {
	&errOpt($createUsage);
    }
} # end subCreate

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# subroutine for adding CA and regular certificates to the database
sub doAdd {
    #print "Doing add\n";
    # make sure the certificate file exists
    if (!defined $opt_certFile) {
	&errOpt($addUsage);
    }
    if (! -e $opt_certFile) {
      error(MSG_FILE_DOES_NOT_EXIST, $opt_certFile);
	exit 3;
    }

    if (defined $opt_CACert && defined $opt_certFile && defined $opt_dataType && defined $opt_label) {
	$args = "aca " . $opt_keyFile . " " . $opt_password . " \"" . $opt_label . "\" " . $opt_certFile . " " . $opt_dataType;
	&runJava($args);
    } 
#    if (defined $opt_Cert) {
#	print "Yes 1\n";
#    }
#    if (defined $opt_certFile) {
#	print "Yes 2\n";
#    }
#    if (defined $opt_dataType) {
#	print "Yes 3\n";
#    }
    elsif (defined $opt_Cert && defined $opt_certFile && defined $opt_dataType) {
	$args = "ac " . $opt_keyFile . " " . $opt_password . " " . $opt_certFile . " " . $opt_dataType;
	&runJava($args);
    } else {
	#print "in else\n";
	&errOpt($addUsage);
    }
}

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# subrotine to delete CA's, Certs and Cert Requests
sub doDel {    
    if (defined $opt_Cert && defined $opt_keyFile && defined $opt_password && defined $opt_label) {
	$args = "dc " . $opt_keyFile . " " . $opt_password . " \"" . $opt_label . "\"";
	&runJava($args);
	return;
    } elsif (defined $opt_CACert && defined $opt_keyFile && defined $opt_password && defined $opt_label) {
	$args = "dca " . $opt_keyFile . " " . $opt_password . " \"" . $opt_label ."\"";
	&runJava($args);
	return;
    } elsif (defined $opt_Req && defined $opt_keyFile && defined $opt_password && defined $opt_label) {
	$args = "dcr " . $opt_keyFile . " " . $opt_password . " \"" . $opt_label ."\"";
	&runJava($args);
	return;	
    }
    else {
	&errOpt($delUsage);
    }
}

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# subrotine to list CA's, Certs and Cert Requests
sub doList {
    if (defined $opt_Cert) {
	$args = "lc " . $opt_keyFile . " " . $opt_password;
	&runJava($args);
    } elsif (defined $opt_CACert) {
	$args = "lca " . $opt_keyFile . " " . $opt_password;
	&runJava($args);
    } elsif (defined $opt_Reqs) {
	$opt_Reqs = 1; # FIX
	$args = "lr " . $opt_keyFile . " " . $opt_password;
	&runJava($args);
    } else {
	&errOpt($listUsage);
    }
}

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# subroutine to import a PKCS file
sub doImport {
    if (defined $opt_PKCS && $opt_certFile && $opt_certPassword) {
	$opt_PKCS = 1;  # remove warning
	if ( -e $opt_certFile) {
	    $args = "ipk " . $opt_keyFile . " " . $opt_password . " " . $opt_certFile . " " . $opt_certPassword;
	    &runJava($args);
	} 
	else {
	  error(MSG_FILE_DOES_NOT_EXIST, $opt_certFile);
	    exit 4;
	}
    }
    else {
	&errOpt($importUsage);
    }
} # end import

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# subroutine to run the constructed command line
# with the correct java environment
sub runJava {
    &setEnv();
    my $myArgs = shift(@_);
    $totalCmd = "";
    if (defined $opt_debug) {
	trace "Running with args:\n\t" . $myArgs . "\n";
    }
    my $totalCmd = $ENV{"JAVA_HOME"} . "/bin/java -classpath " . $ENV{"CLASSPATH"} . " " . 
	$SSLCertMgr . " " . $myArgs;    
    # run it
    #print $totalCmd, "\n";
    if (! defined($opt_noRun)) {
      my ($rc, $stdout, $stderr);
      $rc = runit([$totalCmd], \$stdout, \$stderr);
      if ($rc != 0) {
	trace("command failed; rc=$rc\n$stdout\n$stderr");
      }
    }
    
}


sub errOpt {
    my $errArgs = shift(@_);
    error(MSG_USAGE_ERROR, $0, $errArgs);
    exit 3;
}

# subroutine to set the environment
sub setEnv {
# ----------------------------------------------------------------------
# This is the default Java Home
# ----------------------------------------------------------------------
    if (!exists $ENV{"JAVA_HOME"}) {
	#print "Setting JAVA_HOME\n";
      if (-d "/opt/IBMJava2-122") {
	  $ENV{"JAVA_HOME"} = "/opt/IBMJava2-122";
      }
	elsif (-d "/opt/ibm-jdk-l118") {
	  $ENV{"JAVA_HOME"} = "/opt/ibm-jdk-l118";
	} elsif (-d "/usr/jdk118") {
	  $ENV{"JAVA_HOME"} = "/usr/jdk118";
	} else {
	  error(MSG_CANNOT_FIND_JAVA);
	  exit 3;
	}
    }
    trace("Found java at " . $ENV{"JAVA_HOME"});
    $ENV{"PATH"} = $ENV{"PATH"} . ":" . $ENV{"JAVA_HOME"}."/bin";
# ----------------------------------------------------------------------
# Determine IKEYMAN_HOME
# ----------------------------------------------------------------------
    if (!exists $ENV{"IKEYMAN_HOME"}) {
#	print "Setting IKEYMAN_HOME\n";
	$ENV{"IKEYMAN_HOME"} = "/usr/local/ibm/gsk4";
    }
# ----------------------------------------------------------------------
# Set IKEYMAN environment
# ----------------------------------------------------------------------
    $ENV{"IKEYMAN_CLASS_DIR"}=$ENV{"IKEYMAN_HOME"}."/classes";
    $ENV{"CLASSPATH"}=$ENV{"CLASSPATH"}.":".
	$ENV{"IKEYMAN_CLASS_DIR"}."/gsk4cls.jar:".
	    $ENV{"JAVA_HOME"}."/lib/classes.zip:".":".
		$ENV{"IKEYMAN_CLASS_DIR"}."/cfwk.zip:".":".
		    "/opt/IBMWebAS/servlets/ibmasm.jar";

}
