#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

use Getopt::Std;
use IO::File;

use ASM::Common;
# Don't call commonInit; it'll eat all the arguments...

# This function copies the specified file into a temporary file and returns the name of
# the newly created file.

$prefix = "csmlog";

#GRAB THE COMMAND LINE PARAMETERS -f FILENAME (to copy)
getopts("f:");

if (!$opt_f)
{
  error(MSG_USAGE_ERROR, $0, "$0 -f filename");
  exit(1);
}

if (! -f $opt_f) {
  error(MSG_FILE_DOES_NOT_EXIST, $opt_f);
  exit(1);
}
if (! ($fin = IO::File->new($opt_f, O_RDONLY))) {
  trace("opening file $opt_f: $!");
  error(MSG_COULD_NOT_OPEN_FILE, $opt_f);
  exit(1);
}

# Create temporary file
undef $fout;
    
for ($iteration=0; $iteration < 5; ++$iteration) { # in case of races
  # find existing csmlog files
  %files = ();
  opendirOrDie(TMPDIR, "/tmp");
  %files = map {$_,$_} grep /^$prefix/, readdir TMPDIR;
  closedir TMPDIR;
  
  #find available slot
  for ($i = 0; $files{"$prefix$i"}; $i++){};
  
  if ($fout = IO::File->new("/tmp/$prefix$i", O_RDWR|O_CREAT|O_EXCL)) {
    $name = "/tmp/$prefix$i";
    last;
  }
  # else we lost a race with another process trying to
  # use the same name; go around again.
}

if (!$fout)
{
  trace("couldn't find an unused temp file name in /tmp/$prefix*");
  error(MSG_COULD_NOT_CREATE_TEMPORARY_FILE);
  exit(1);
}

# Read in the log file

$i=0;
if ($fin) {
  while (<$fin>) {
    # Back-whack all the quotation marks
    s/"/\"/gi;  #"

    # For CSM audit log remove localhost
    if ($opt_f eq "csmAudit.log")
    {
      s/createAuditEntry://gi;
      s/^(\w+ \d+ [\d\:]+) [\w\d]+/$1/;
    }

    $lines[$i++]=$_; 
  }
  close($fin);
}

# Reverse log into a temporary file

for ($j = $i-1; $j >= 0; $j--)
{
  print $fout $lines[$j];
}
close($fout);

print "$name $i";
