#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# monitorASM.pl - Check if the machine is up and if data can be
#                 pulled from Web server.
#
# Wayne Gao, <waynegao@us.ibm.com> (29 Octobe 1999)
#

use strict;

my ($eSP_BIN, $checkweb, $checkapp, $date, $rc, $stdout, $stderr, $hostname);

$eSP_BIN = "/opt/CSM/scripts";

use ASM::Common;
use ASM::Exec;
use ASM::Log;

require "$eSP_BIN/common_install.pl";

setFacility('initsys');

commonInit([],"");

$checkweb = "$eSP_BIN/checkWeb.pl";
$checkapp = "$eSP_BIN/monitorApp.pl";

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;

   trace("$exitmsg");
   trace("exit $exitrc");
   $date = `date|tr -d "\n"`;
   trace("++++ Stop: monitorASM.pl ($date) ++++");
   exit $exitrc;

}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}                                          

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

trace("+++ monitorASM.pl started +++");

# Run some basic commands
$rc = runit(["/bin/hostname"], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of hostname command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: hostname comamnd failed, exit", 3);
}

$rc = runit(["/bin/uname -a"], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of uname command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: uname comamnd failed, exit", 3);
}

$hostname = `hostname`;
chomp($hostname);
$rc = runit(["/bin/ping -c1 $hostname"], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of ping command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: ping comamnd failed, exit", 3);
}

# Check some processes

#  See if syslod is Running.
$rc = runit(["ps -ef|grep syslogd|grep -v grep"], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of syslogd ps command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: syslogd is not running, exit", 3);
}

#  See if crond is Running.
$rc = runit(["ps -ef|grep crond|grep -v grep"], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of crond ps command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: crond is not running, exit", 3);
}

#  See if inetd is Running.
$rc = runit(["ps -ef|grep inetd|grep -v grep"], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of inetd ps command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: inetd is not running, exit", 3);
}

#  See if WebSphere Application Server is Running
#  Note, The wdith output for ps must be increased to get enough detail to identify App Server
#        If WebAppServer is changed or if java is changed, this check will probably need to be changed.
$rc = runit(["ps -ef --width 200|grep \"/opt/IBMJava2-122/jre/bin/exe/java com.ibm.servlet.engine.ServletEngine\"|grep -v grep"], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of IBMWebAS ps command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: IBMWebAS is not running, exit", 3);
}

#  See if HTTP is Running
$rc = runit(["ps -ef --width 200|grep /opt/IBMHTTPServer/bin/httpd|grep -v grep"], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of httpd ps command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: httpd is not running, exit", 3);
}

# See if Web pages can be accessed
trace("Run $checkweb.");
$rc = runit(["$checkweb"], \$stdout, \$stderr);
if (-e "$eSP_BIN/temp")
{
   $rc = runit(["rm -f $eSP_BIN/temp"], \$stdout, \$stderr);
}
if ($rc != 0)
{
   error(MSG_ASM_NOT_RUNNING_NORMALLY); 
   trace("Output of web access command: " . $stdout . $stderr);
   &cleanup_exit("ERROR: Web process is running, but Page can't be pulled out.", 3);
}

# Check the Appliance Application
trace("Run $checkapp.");
$rc = runit(["$checkapp"], \$stdout, \$stderr);
if ($rc != 0)
{
   trace("Output of Appliance Application check command: " . $stdout . $stderr);
   error(MSG_APPLICATION_NOT_RUNNING_NORMALLY); 
   &cleanup_exit("ERROR: Application doesn't seem to be running normally.", 3);
}
else
{
   audit(MSG_ASM_RUNNING_NORMALLY); 
   &cleanup_exit("ASM applicance and application running normally.", 0);
}

