#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# MonitorCMS.sh - Check if the machine is up and if data can be
#                 pulled from Web server.
#
# Wayne Gao, <waynegao@us.ibm.com> (29  Octobe 1999)
#

use ASM::Common;
use ASM::Exec;

setFacility('initappl');

$checkweb = "$script_path/checkWeb.pl";
                                                     
sub sig_handler
{
   local($sig) = @_;
   error(MSG_CAUGHT_SIGNAL, $sig);
   exit(1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

#  Lots of assumptions must be made to check the HTTP Server Application.
#  Since the user can define their own configuration file, change the
#  Port numbers and change the pid files, its difficult to have a
#  bulletproof method for checking the HTTP Server.
$appurl = ":80";  #  Assuming HTTP configuration uses Port 80
$pidfile = "/opt/IBMHTTPServer/logs/httpd.pid";

if (! -f $pidfile)
{
  trace("web server pid file $pidfile not found");
  error(MSG_WEBHOST_NOT_WORKING);
  exit(1);
}
openOrDie(PID, $pidfile);
chomp($serverpid = <PID>);
my ($rc,$stdout,$stderr);
$rc = runit(["ps", "-h", "-p", $serverpid], \$stdout, \$stderr);
if ($rc != 0)
{
  trace("web server process with pid $serverpid not found: rc=$rc\n$stdout\n$stderr");
  error(MSG_WEBHOST_NOT_WORKING);
  exit(1);
}

# Pull some pages from Web

$rc = runit([$checkweb, $appurl], \$stdout, \$stderr);

if ($rc != 0)
{
  trace("test of web server failed; rc=$rc\n$stdout\n$stderr");
  error(MSG_WEBHOST_NOT_WORKING);
  exit(1);
}
else
{
  audit(MSG_WEBHOST_OKAY);
  exit(0);
}
