#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# nameservers.pl - Sets and gets the resolv.conf file.
#
# Paul Schroeder <paulsch@us.ibm.com> (28 September 1999)
#
#


@nameservers_opts = ("list", "add", "search:s@", "ns|nameserver:s@", "append",
		    "clearns", "clearsearch");
$nameservers_usage = <<EOS;
nameservers.pl --list --ns
nameservers.pl --list --search
nameservers.pl --add [ --ns=NAMESERVER ... ] [ --search=DOMAIN --search=... ] [ --append ]
nameservers.pl --clearns
nameservers.pl --clearsearch
EOS

use ASM::Common;
commonInit(\@nameservers_opts, $nameservers_usage);
setFacility('netcfg');
$resolv_file = "/etc/resolv.conf";

if($opt_list) {
	if(@opt_ns) {
		@servers = &getNSList();
		for(@servers) {
			print $_ . "\n";
		}#for
	} elsif(@opt_search) {
		@search_list = &getSearchList();
		for(@search_list) {
			print $_ . "\n";
		}#for
	} else {
		&printUsage();
	}#if-else-elsif
} elsif($opt_add) {
	&setSearchList($opt_append, @opt_search) if @opt_search;
	&setNSList($opt_append, @opt_ns) if @opt_ns;
} elsif ($opt_clearns) {
  &setNSList(0);
} elsif ($opt_clearsearch) {
  &setSearchList(0);
} else {
	&printUsage();
}#if-elsif-else

exit(0);


################################
# Lookout!  Subroutines below! #
################################

# Gets a list the list of name servers from the resolv.conf file and returns
# a scalar array containing the names.
sub getNSList {
	my @ns_list = ();

	openOrDie(RESOLV, "<$resolv_file");
	# Cycle through resolv.conf pushing each NS onto ns_list.
	while(<RESOLV>) {
		if(/^nameserver\s+(.*)\s+$/) {
			push(@ns_list, $1);
		}
	}
	close(HOSTS);

	return @ns_list;
}#getNSList

# Sets the new nameserver list.  Append to the list if --append was specified.
sub setNSList {
	my $append = shift(@_);
	my @ns_list = @_;
	my @new_ns_list = ();

	for(@ns_list) {
	     if($_)
   	     {
	       if (!goodIP($_)) { # require IP address for DNS -- hostname doesn't make sense here
		 error(MSG_HOSTS5, $_); # invalid IP address
		 exit(5);
	       }
	     }
	}#for

	if($append) {
		@new_ns_list = &getNSList();
		push(@new_ns_list, @ns_list);
	} else {
		@new_ns_list = @ns_list;
	}#if-else
	&writeResolv(join(" ", &getSearchList), @new_ns_list);

	audit(MSG_NAME_SERVERS_UPDATED, join(' ',@new_ns_list));
}#setNSList

# Writes the search list and nameservers to the resolv.conf file.
sub writeResolv {
	my $search_list = shift(@_);
	my @ns_list = @_;

	openOrDie(RESOLV, ">$resolv_file");
	print RESOLV "search $search_list\n" if $search_list;
	for(@ns_list) {
		print RESOLV "nameserver ". $_ . "\n";
	}#for
	close(RESOLV);
}#writeResolv

# Sets the search line in resolv.conf appending if --append was specified.
sub setSearchList {
	my $append = shift(@_);
	my @search_list = @_;
	my @new_search_list = ();

	for(@search_list) {
	  unless (goodDomain($_)) {
	    error(MSG_DOMAIN_NOT_VALID, $_);
	    exit(6);
	  }
	}#for

	if($append) {
		@new_search_list = &getSearchList();
		push(@new_search_list, @search_list);
	} else {
		@new_search_list = @search_list;
	}#if-else
	&writeResolv(join(" ", @new_search_list), &getNSList);

	audit(MSG_DOMAIN_SEARCH_LIST_UPDATED, join(' ', @new_search_list));
}#setSearchList

# Gets the domains from the search line of resolv.conf.
sub getSearchList {
	my @search_list = ();

	openOrDie(RESOLV, "<$resolv_file");
	while(<RESOLV>) {
		if(/^search\s+(.*)\s+$/) {
			@search_list = split(/\s+/, $1);
			last;
		}#if
	}#while
	close(RESOLV);

	return @search_list;
}#getSearchList

