#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# network.pl - Sets and gets the network info for the machine.
#
# Paul Schroeder <paulsch@us.ibm.com> (30 September 1999)
#
#

#
# To remove the default route (gateway), do
#   network.pl --set --gateway=none
#

@network_opts = ("get", "set", "all", "networking=s", "forward_ipv4=s", "hostname=s", "domainname=s", "gateway=s", "gatewaydev=s", "nisdomain=s");
$network_usage = <<"EOS";
network.pl --get [ --all ] [ networking ] [ forward_ipv4 ] [ hostname ] [ domainname ] [ gateway ] [ gatewaydev ] [ nisdomain ]
network.pl --set  [ --networking=[yes|no] ] ] [ --forward_ipv4=[yes|no] ] [ --hostname=HOSTNAME ] [ --domainname=DOMAINNAME ] [ --gateway=GATEWAY ] [ --gatewaydev=GATEWAYDEV ] [ --nisdomain=NISDOMAIN ]
EOS

use ASM::Common;
use ASM::Exec;
commonInit(\@network_opts, $network_usage);
setFacility('netcfg');

$network_file = "/etc/sysconfig/network";
use DB_File;
use strict;
BEGIN { $^W = 1; }
use vars qw! $opt_get $opt_all $opt_set $opt_networking $opt_forward_ipv4 $opt_hostname
  $opt_domainname $opt_gateway $opt_gatewaydev $opt_nisdomain
  $sep
  $network_file
  $hostname_file
  $auditcmd
  $device_file
  $hosts_file
  $script_path
  !;

use constant NONE => "none";

my @network_parms = ('NETWORKING', 'FORWARD_IPV4', 'HOSTNAME', 'DOMAINNAME', 'GATEWAY', 'GATEWAYDEV', 'NISDOMAIN');

if($opt_get) {
	if($opt_all) {
	    my %network_values = &getNetwork();
	    print join($sep, @network_values{@network_parms}) . "\n";
	} else {
		# All non-option command line options get put int @ARGV
		my $optstring = join($sep, &getNetworkOptions(@ARGV));
		print $optstring . "\n" if $optstring ne "\n";
	}#if-else
} elsif($opt_set) {
	&setNetwork($opt_networking, $opt_forward_ipv4, $opt_hostname, $opt_domainname, $opt_gateway, $opt_gatewaydev, $opt_nisdomain);
} else {
	&printUsage();
}#if-elsif-else

exit(0);


################################
# Lookout!  Subroutines below! #
################################

# Gets the current network info.  Remember $. is a perl special var which
# is the line number of the last file handle read.
sub getNetwork {
	my %network;
        #  Initialize since some elements may not exist in the network file.
        @network{@network_parms} = ("") x @network_parms;

	openOrDie(NETWORK, "<$network_file");
	while(<NETWORK>) {
		chop;  # remove newline from end
		my ($key, $value) = split(/=/);
		$network{uc($key)} = $value;
	}#while
	close(NETWORK);

	return %network;
}#getNetwork

# Get the network info from the file.
sub getNetworkOptions {
	my @get_options = @_;
	my %opthash = ();
	my @options = ();

	# For each option specified on the command line we create an entry
	# in our assoc array for comparison later.
	for(@get_options) {
		$opthash{$_} = 1;
	}#for

	openOrDie(NETWORK, "<$network_file");
	# Get the key-value pair from each line and push value onto @options
	# if the key is in $opthash.
	while(<NETWORK>) {
		chop;
		my ($key, $value) = split(/=/);
		$key =~ tr/[A-Z]/[a-z]/;
		push(@options, $value) if($opthash{$key});
	}#while
	close(NETWORK);

	return reverse(@options);
}#getNetworkOptions

# Puts the specified data into the network config file.
sub setNetwork {
	my @new_network = @_;
	my %network = &getNetwork();
	my @network_parms = ('NETWORKING', 'FORWARD_IPV4', 'HOSTNAME', 'DOMAINNAME', 'GATEWAY', 'GATEWAYDEV', 'NISDOMAIN');

#	die "Bad hostname $network[2]!" if $new_network[2] && !&goodHostname($network[2]);
#	die "Bad domain $network[3]!" if $new_network[3] && !&goodDomain($network[3]);
#	die "Bad gateway $network[4]!" if $new_network[4] && !&goodIP($network[4]);
	(error(MSG_HOSTNAME_NOT_VALID, $new_network[2]) and exit(5)) if $new_network[2] && !&goodHostname($new_network[2]);
	(error(MSG_DOMAIN_NOT_VALID, $new_network[3]) and exit(6)) if $new_network[3] && !&goodDomain($new_network[3]);
	(error(MSG_IP_ADDRESS_NOT_VALID, $new_network[4]) and exit(7)) 
	  if $new_network[4] && !&goodIP($new_network[4]) && $new_network[4] ne NONE;

	my ($rc,$stdout,$stderr);

	my $net_cmd = "";
	if($new_network[0] && $new_network[0] ne $network{'NETWORKING'}) {
		$net_cmd = "/etc/rc.d/init.d/network ";
		$net_cmd .= ($new_network[0] =~ /^yes/i) ? "start" : "stop";
		runit([$net_cmd], \$stdout, \$stderr);
	}#if
	if($new_network[1] && $new_network[1] ne $network{'FORWARD_IP4'}) {
		my $ipf = $new_network[1] =~ /^yes$/i ? "1" : "0";
		openOrDie(OUT, ">/proc/sys/net/ipv4/ip_forward");
		print OUT $ipf, "\n";
		close(OUT);
	}#if
	if($new_network[2] && $new_network[2] ne $network{'HOSTNAME'} || $new_network[3] && $new_network[3] ne $network{'DOMAINNAME'}) {
		my $net_host = ($new_network[2]) ? $new_network[2] : $network{'HOSTNAME'};
		my $net_dom = ($new_network[3]) ? $new_network[3] : $network{'DOMAINNAME'};
		my $junk;
		($net_host, $junk) = split(/\./, $net_host, 2);
		$net_host .= "." . $net_dom if $net_dom;
		$new_network[2] = $net_host;
#       place our new hostname into /etc/hosts
		modifyEtcHosts($network{'HOSTNAME'},$net_host);
		runit(["hostname", $net_host],\$stdout,\$stderr);
		# Make sure we write our new host name to HOSTNAME
		openOrDie(HOSTNAME, ">$hostname_file");
		print HOSTNAME $new_network[2] . "\n";
		close(HOSTNAME);
		# Update webservers
		runit(["$script_path/configureWebservers.pl"],\$stdout,\$stderr);

	}#if
	if($new_network[4] || $new_network[5]) {
		my $net_gw = ($new_network[4]) ? $new_network[4] : $network{'GATEWAY'};
		my $net_gwd = ($new_network[5]) ? $new_network[5] : $network{'GATEWAYDEV'};
		if ($net_gw eq NONE) {
		  $network{'GATEWAYDEV'} = NONE;
		}
		runit(["route del default"],\$stdout,\$stderr);
		if ($net_gw ne NONE) {
		  runit(["route add default gw $net_gw metric 1 $net_gwd"],\$stdout,\$stderr);
		}
	}#if

	# Assign new value to network if value specified
	foreach my $i ( 0 .. $#network_parms) {
		$network{$network_parms[$i]} = $new_network[$i] if $new_network[$i];
	}#for

	# Write new network file
	openOrDie(NETWORK, ">$network_file");
	my $parms = "";
	foreach my $i (keys(%network))
	{
	  next if $network{$i} eq NONE;
	  print NETWORK $i . "=" . $network{$i} . "\n";
	  $parms = "$parms $i=$network{$i}";
	}
	close(NETWORK);

	runit([$net_cmd],\$stdout,\$stderr) if $net_cmd;
	
	audit(MSG_NETWORK_CONFIGURATION_UPDATED, $parms);
}#setNetwork
sub modifyEtcHosts{
	my $old_hostname = shift(@_);
	my $new_hostname = shift(@_);

	my $device = "eth0";
        my %network = ();
        return if ! -e "$device_file$device";
        openOrDie(DEVICE, "<$device_file$device");
        # Read each line getting the key-value pair.
        # $. is the perl special var which equals the last line number read from
        # the last file handle read.
        while(<DEVICE>) {
	    chomp;  # remove newline from end
	    my ($key, $value) = split(/=/);
	    $network{uc($key)} = $value;
	}#while
	close(DEVICE);
	my $ipaddress = $network{'IPADDR'};

	# open /etc/hosts for read/write to edit hostname entry
	my @file;
	tie (@file, 'DB_File', $hosts_file, O_RDWR, 0644, $DB_RECNO) 
          or (trace("opening $hosts_file: $!") 
	    and error(MSG_COULD_NOT_OPEN_FILE, $hosts_file)
	      and exit(5));
	my $i = 0;
        my $found = 0;

	while (defined $file[$i]) {

        	$_ = $file[$i];

		#find the current hostname line
		if (/$ipaddress\s+/){
		    $found = 1;
	            last;
        	}
        	$i++;

	} #while

	
	if ($found) {
	    $file[$i] = $ipaddress. "\t\t".$new_hostname;
	    untie @file;
	} else {  # no hostname entry found just append it to the end with ipaddress of eth0
	    untie @file;
	    openOrDie(HOSTS, ">>$hosts_file");
	    print HOSTS $ipaddress. "\t" . $new_hostname . "\n";
	    close(HOSTS);
	}
}
