#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

use ASM::Common;
use ASM::Exec;

commonInit([],"");
setFacility('backcfg');

use strict;
BEGIN { $^W = 1; }

use vars qw/
  $backup_filename
  $postrestore_script_pattern
/;

my ($rc, $stdout, $stderr);

if (-f $backup_filename) {  
  chdirOrDie("/");
  $rc = runit(["tar", "-Pxvpf", $backup_filename], \$stdout, \$stderr);
  if ($rc != 0) {
    trace("error restoring archive: rc=$rc, " . $stdout . $stderr);
    error(MSG_E53);		# restore configuration failed
    exit(3);
  }
} else {
  trace("no backup file $backup_filename");
  error(MSG_E62);		# there is no backup
  exit(62);
}

# Run any post-restore scripts
my @scripts = glob($postrestore_script_pattern);
foreach my $script (@scripts) {
  if (-x $script) {
    trace("running $script\n");
    $rc = runit([$script], \$stdout, \$stderr);
    if ($rc != 0) {
      trace("Script $script failed with rc=$rc:\n$stdout\n$stderr");
    }
  }
  else {
    trace "script $script is not executable\n";
  }
}

audit(MSG_E52);
exit(0);
