#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# route.pl - Sets and gets the static-routes file.
#
# Paul Schroeder <paulsch@us.ibm.com> (12 October 1999)
#
#


@route_opts = ("add", "get", "del", "type=s", "netmask=s", "gw|gateway=s");
$route_usage = <<"EOS";
route.pl --add DEVICE TARGET --type=[ net | host ] [ --netmask=NETMASK ] [ --gw=GATEWAY ]
route.pl --del DEVICE TARGET --type=[ net | host ] [ --netmask=NETMASK ] [ --gw=GATEWAY ]
route.pl --get [ TARGET ] [ --sep=SEP ]
EOS
use ASM::Common;
use ASM::Exec;
setFacility('netcfg');
commonInit(\@route_opts, $route_usage);
$route_file = "/etc/sysconfig/static-routes";

if($opt_add) {
	$device = shift(@ARGV);
	$target = shift(@ARGV);
	&logUsage() if !$target || !$device || $opt_type !~ /^(net|host)$/i;
	addRoute($device, $target, $opt_type, $opt_netmask, $opt_gw);
} elsif($opt_get) {
	$target = shift(@ARGV);
	@routes = &getRoute($target);
	for(@routes) {
		print $_ . "\n";
	}#for
} elsif($opt_del) {
	$sep = ",";
	$device = shift(@ARGV);
	$target = shift(@ARGV);
	&delRoute($device, $target, $opt_type, $opt_netmask, $opt_gw);
} else {
	&logUsage();
}#if-elsif-elsif-else

exit(0);

################################
# Lookout!  Subroutines below! #
################################

sub delRoute {
	my $device = shift(@_);
	my $target = shift(@_);
	my $type = shift(@_);
	my $netmask = shift(@_);
	my $gw = shift(@_);
	my @routes = &getRoute();

	my $route_cmd = "$type";
	goodIPOrDie($target);
	$route_cmd .= " $target";
	if($netmask) {
	  goodIPOrDie($netmask);
		#die "Invalid netmask $netmask!" if !&goodIP($netmask);
		$route_cmd .= " netmask $netmask";
	}#if
	if($gw) {
	  goodIPOrDie($gw);
		#die "Invalid gateway $gw!" if !&goodIP($gw);
		$route_cmd .= " gw $gw";
	}#if
	$rc = runit(["route del -$route_cmd $device"], \$stdout, \$stderr);
	if ($rc) {
	  trace("route del failed: rc=$rc\n$stdout\n$stderr");
	  error(MSG_ERROR_DELETING_ROUTE);
	  exit(3);
	}

	openOrDie(ROUTE, ">$route_file");
	for(@routes) {
		($o_device, $o_type, $o_target, $o_netmask, $o_gw) = split(/$sep/);
		$route_entry = "$o_device $o_type $o_target";
		$route_entry .= " netmask $o_netmask" if $o_netmask;
		$route_entry .= " gw $o_gw" if $o_gw;
		if($route_entry ne "$device $route_cmd") {
			print ROUTE $route_entry . "\n";
		}#if
	}#for
	close(ROUTE);
	
	$def = "$type $target";
	$def = "$def $netmask" if $netmask;
	$def = "$def $gw" if $gw;

	audit(MSG_DELETED_ROUTE, $def);
}#delRoute

sub getRoute {
	my $my_target = shift(@_);
	my @routes;

	if (-f $route_file) {	# maybe no file if no routes
	  openOrDie(ROUTE, "<$route_file");
	  while(<ROUTE>) {
	    chop;
	    undef($netmask);
	    undef($gw);
	    
	    ($device, $type, $target, $arg1, $val1, $arg2, $val2) = split;
	    if($arg1 eq "netmask") {
	      $netmask = $val1;
	    } elsif($arg1 eq "gw") {
	      $gw = $val1;
	    }			#if-elsif
	    if($arg2 eq "netmask") {
	      $netmask = $val2;
	    } elsif($arg2 eq "gw") {
	      $gw = $val2;
	    }			#if-elsif
	    push(@routes, join($sep, $device, $type, $target, $netmask, $gw)) if !$my_target || $my_target eq $target;
	  }#while
	  close(ROUTE);
	}

	return @routes;
}#getRoute

sub addRoute {
	my $device = shift(@_);
	my $target = shift(@_);
	my $type = shift(@_);
	my $netmask = shift(@_);
	my $gw = shift(@_);

	my $route_cmd = "route add -$type";
	goodIPOrDie($target);
	#die "Invalid target IP $target" if !&goodIP($target);
	$route_cmd .= " $target";
	if($netmask) {
	  goodIPOrDie($netmask);
		#die "Invalid netmask $netmask" if !&goodIP($netmask);
		$route_cmd .= " netmask $netmask";
	}#if
	if($gw) {
	  goodIPOrDie($gw);
		#die "Invalid gateway $gw" if !&goodIP($gw);
		$route_cmd .= " gw $gw";
	}#if
	$route_cmd .= " $device";
	#die "Can't add route '$route_cmd' $!" if system($route_cmd);
	$rc = runit([$route_cmd], \$stdout, \$stderr);
	if ($rc) {
	  trace("Error adding route: $route_cmd rc=$rc\n$stdout\n$stderr");
	  error(MSG_ERROR_ADDING_ROUTE);
	  exit(3);
	}

	openOrDie(ROUTE, ">>$route_file");
	print ROUTE "$device $type $target";
	print ROUTE " netmask $netmask" if($netmask);
	print ROUTE " gw $gw" if($gw);
	print ROUTE "\n";

	close(ROUTE);

	$def = "$type $target";
	$def = "$def $netmask" if $netmask;
	$def = "$def $gw" if $gw;
	audit(MSG_ADDED_ROUTE, $def);
}#addRoute

