#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
# setup.pl - Config wizard
#
# Paul Schroeder <paulsch@us.ibm.com> (17 November 1999)
#

my $eSP_BIN = "/opt/CSM/scripts";
@setup_opts = ("nopassword");
$setup_usage = "setup.pl [ --nopassword ]\n";
use POSIX qw(:termios_h);
use ASM::Common;
use ASM::Exec;
my ($rc,$stdout,$stderr);
setFacility('initsys');
$setup_device = "eth0";

changeLocale("en_US");

commonInit(\@setup_opts, $setup_usage);

# Prototypes
sub doPrompt ($$$$);
sub runOrDie ($);
sub printMenu ($$$);

#@net_options = ('Host Name', 'Domain Name', 'IP Address', 'Netmask', 'Gateway (optional)', 'Primary DNS (optional)');
#@net_data = ('', '', '', '', '', '');
push(@net_options, getFormattedMessage(MSG_SETUP_OPTION_0)); # hostname
push(@net_options, getFormattedMessage(MSG_SETUP_OPTION_1)); # domain name
push(@net_options, getFormattedMessage(MSG_SETUP_OPTION_2)); # ip address
push(@net_options, getFormattedMessage(MSG_SETUP_OPTION_3)); # network mask
push(@net_options, getFormattedMessage(MSG_SETUP_OPTION_4)); # gateway (optional)
push(@net_options, getFormattedMessage(MSG_SETUP_OPTION_5)); # primary dns (optional)
push(@net_options, getFormattedMessage(MSG_SETUP_OPTION_6)); # Language 

use constant HOSTNAME_OPTION => 0;
use constant DOMAINNAME_OPTION => 1;
use constant IPADDRESS_OPTION => 2;
use constant NETMASK_OPTION => 3;
use constant GATEWAY_OPTION => 4;
use constant DNS_OPTION => 5;	       
use constant LANGUAGE_OPTION => 6;
use constant OLD_PASSWORD_OPTION => 7;
use constant NEW_PASSWORD_OPTION => 8;
use constant VERIFY_PASSWORD_OPTION => 9;

@locale_list = ();
@locale_list = split("\n", `$eSP_BIN/queryLocales.pl`);
trace("Locales Found on Appliance are: @locale_list");

@net_data = ();
$title = getFormattedMessage(MSG_WELCOME_TO_SETUP) . "\n"; 

# Compute length of longest option prompt
$lop = 0;
for (@net_options) {
	$lop = length if length > $lop;
}

&printTitle();

if (!$opt_nopassword) {
	&getPassword($lop);
	&continue();
	&printTitle();
}

$i = 0;
for (@net_options) {
  push(@net_data, &doPrompt($_, '', $lop,$i++));
}

&continue();

$option = "";
while ($option ne "9" || $option =~ /^\s*$/) {
	&printMenu(\@net_options, \@net_data, $lop);
	chomp($option = <STDIN>);
	if($option !~ /^\s*$/ && $option >= 0 && $option <= $#net_options) {
		$net_data[$option] = &doPrompt($net_options[$option], $net_data[$option], $lop, $option);
	}#if
}#while

# Okay...  The user is done, let's go to work...

# NOTE that all of our utilities will write error
# messages to stderr, so if they fail, all we have
# to do here is grab those messages and print them.

$cmd = "/etc/rc.d/init.d/arpfind stop";
run($cmd);			# ignore result - might not have been running

# Disable arpfind for future boots
$cmd = "/opt/CSM/scripts/arpfind.pl --disable";
runOrDie($cmd);

$cmd = "$script_path/device.pl --set $setup_device --ipaddr=$net_data[2] --netmask=$net_data[3]";
runOrDie($cmd);

if ($net_data[4]) {
  $cmd = "$script_path/network.pl --set --hostname=$net_data[0] --domainname=$net_data[1] --gateway=$net_data[4] --gatewaydev=$setup_device";
} else {
  $cmd = "$script_path/network.pl --set --hostname=$net_data[0] --domainname=$net_data[1]";
}
runOrDie($cmd);

#print "Updating hosts file...";
$cmd = "$script_path/hosts.pl --set --ip=$net_data[2] --hostname=$net_data[0]";
runOrDie($cmd);

if($net_data[5]) {
	#print "Setting Primary DNS server...";
	$cmd = "$script_path/nameservers.pl --add --ns=$net_data[5] --search=$net_data[1]";
	#print "cmd=$cmd";
	runOrDie($cmd);
	#die "Could not set up Primary DNS server!: $!" if execute($cmd);
	#print " Done...\n\n"
} else {
  # Set search path anyway
  if ($net_data[1]) {
    $cmd = "$script_path/nameservers.pl --add --search=$net_data[1]";
    #print "cmd=$cmd";
    runOrDie($cmd);
  }

  print getFormattedMessage(MSG_PRIMARY_DNS_NOT_SET), "\n";
}#if-else

#print "Setting up SSL...";
$cmd = "$script_path/createSelfSignedCert.pl";
#"createCertificate.sh SSLpassword $net_data[2]";
runOrDie($cmd);
#die "Could not set up SSL!: $!" if execute($cmd);
#print " Done...\n\n";

#print "Disabling arpfind...";
$cmd = "$script_path/arpfind.pl --disable";
#print "cmd=$cmd";
runOrDie($cmd);
#die "Could not disable arpfind!: $!" if execute($cmd);
#print " Done...\n\n";

#  Setting Language
chomp($net_data[6]);
$cmd = "$script_path/setlocaleHTTP.pl --locale=$net_data[6]";
runOrDie($cmd);
$cmd = "perl -p -i -e 's/^Language=.*/Language=$net_data[6]/' $setup_data_file";
runOrDie($cmd);

$cmd = "perl -p -i -e 's/^Initialized=.*/Initialized=1/' $setup_data_file";
runOrDie($cmd);

#print "\nRebooting the machine will allow your new changes to take effect.\n\n";
#print "Reboot now [Y]? ";
print getFormattedMessage(MSG_REBOOT_MESSAGE), "\n";
print getFormattedMessage(MSG_REBOOT_PROMPT), "\n";
print "[1]? ";
chop($input = <STDIN>);
$input = "1" if $input =~ /^\s*$/;
if($input =~ /^1/i) {
  print getFormattedMessage(MSG_REBOOTING_MESSAGE);
  $cmd = "$script_path/shutdown.pl --reboot";
  runOrDie($cmd);
}#if

exit(0);

# Turn off input echoing
sub echo_off {
  my $termios = new POSIX::Termios;
  if ($termios->getattr(0)) {
    my $c_lflag = $termios->getlflag;
    my $echo = ECHO | ECHOK | ICANON;
    $c_lflag = $c_lflag & ~$echo;
    $termios->setlflag($c_lflag);
    $termios->setattr(0, TCSANOW);
  }
}
# Turn on input echoing
sub echo_on {
  my $termios = new POSIX::Termios;
  if ($termios->getattr(0)) {
    my $c_lflag = $termios->getlflag;
    my $echo = ECHO | ECHOK | ICANON;
    $c_lflag = $c_lflag | $echo;
    $termios->setlflag($c_lflag);
    $termios->setattr(0, TCSANOW);
  }
}

END { echo_on(); }		# don't leave input messed up

sub getPassword {
	my $lop = shift;

	print getFormattedMessage(MSG_SET_ROOT_PASSWORD), "\n";
	echo_off();
	do {
		$old_pass = &doPrompt(getFormattedMessage(MSG_OLD_PASSWORD), '', $lop, OLD_PASSWORD_OPTION);
		$new_pass = &doPrompt(getFormattedMessage(MSG_NEW_PASSWORD), '', $lop, NEW_PASSWORD_OPTION);
		$new_pass2 = &doPrompt(getFormattedMessage(MSG_NEW_PASSWORD_AGAIN), '', $lop, VERIFY_PASSWORD_OPTION);
		if ($new_pass ne $new_pass2) {
		  print "\n", getFormattedMessage(MSG_E14),"\n"; # don't match
		}
		else {
		  print "\n";
		}
	} while($new_pass ne $new_pass2 || execute("chngrootpass.pl --oldpassword=$old_pass --password=$new_pass") != 0);
	echo_on();
	#print "\n\nThe 'root' password has been set.\n\n";
}#getPass

sub printMenu ($$$) {
  my ($options_ref, $data_ref, $lop) = @_;
  my (@options) = @$options_ref;
  my (@data) = @$data_ref;
  my ($pad) = $lop;

#	my $opl = $#_;
#	my @options = @_[0 .. ($opl/2-1)];
#	my @data = @_[(($opl)/2) .. ($opl-1)];
#	my $pad = $_[$#_];

	&printTitle();
	#print "opl=$opl\n";
	#print "Please verify the data and make any necessary changes.\n\n";
	print getFormattedMessage(MSG_VERIFY_AND_CHANGE),"\n";
	foreach $i (0 .. $#options) {
		print $i . ") ";
		print $options[$i];
		print " " x ($pad - length($options[$i]));
                if ($i == LANGUAGE_OPTION)
                {
		   print " : " . getFormattedMessage(eval("MSG_LANG_" . uc($data[$i]))) . "\n";
                }
                else
                {
		   print " : " . $data[$i] . "\n";
                }
	}#foreach

	#print "\n9) Exit and complete setup.\n\n";
	print "\n9) " . getFormattedMessage(MSG_EXIT_AND_COMPLETE_SETUP) . "\n";
	#print "Enter number to change --> ";
	print getFormattedMessage(MSG_ENTER_NUMBER_TO_CHANGE); # no newline here
}#printMenu

sub goodLanguage()
{
   my ($selected) = @_;
   foreach $i (0 .. $#locale_list)
   {
      if ($selected == $i)
      {
         return(1);
      }
   }
   return(0); 
}

sub doPrompt ($$$$) {
  my ($prompt, $old_data, $pad, $option) = @_;
  
  $data = "";
  
  if ($option == LANGUAGE_OPTION)
    {
      #  Build a menu prompt for the Language selection
      $prompt = getFormattedMessage(MSG_SELECT_LANGUAGE) . "\n";
      foreach $i (0 .. $#locale_list)
	{
	  $prompt = $prompt . $i . ") ";
	  $prompt = $prompt . getFormattedMessage(eval("MSG_LANG_" . uc($locale_list[$i]))) . "\n";
	}
      $prompt = $prompt .  getFormattedMessage(MSG_ENTER_NUMBER_TO_SELECT);
      $old_data = getFormattedMessage(eval("MSG_LANG_" . uc($old_data))) if $old_data;
    }
  do {
    $error = 0;
    print "\n" . " " x ($pad - length($prompt)) . $prompt;
    print " [$old_data]" if $old_data;
    print " --> ";
    chop($data = <STDIN>);
    $data = $old_data if $data =~ /^\s*$/;
    # paulsch: In an effort to save time, this hack was done...
    if(($option==HOSTNAME_OPTION) && !goodHostname($data)) {
      print getFormattedMessage(MSG_HOSTNAME_NOT_VALID, $data),"\n";
      $data = "";
      $error = 1;
    } elsif (($option==DOMAINNAME_OPTION) && !goodDomain($data)) {
      print getFormattedMessage(MSG_DOMAIN_NOT_VALID, $data),"\n";
      $data = "";
      $error = 1;
    } elsif (($option==DNS_OPTION) && (!goodIP($data))) {
      if ($data !~ /^\s*$/) { # Empty is okay, non-empty and bad is not
	print getFormattedMessage(MSG_IP_ADDRESS_NOT_VALID, $data),"\n";
	$error = 1;
      }				#if
      $data = "";
    } elsif ((($option==IPADDRESS_OPTION) || ($option==NETMASK_OPTION)) && !goodIP($data)) {
      print getFormattedMessage(MSG_IP_ADDRESS_NOT_VALID, $data),"\n";
      $data = "";
      $error = 1;
    } elsif (($option==GATEWAY_OPTION) && !goodIP($data)) {
      if ($data !~ /^\s*$/) { # Empty is okay, non-empty and bad is not
	print getFormattedMessage(MSG_IP_ADDRESS_NOT_VALID, $data),"\n";
	$data = "";
	$error = 1;
      }
    }
    elsif (($option==LANGUAGE_OPTION) && !&goodLanguage($data) && $data ne "")
      {
	print getFormattedMessage(MSG_LANGUAGE_NOT_VALID, $data),"\n";
	$data = "";
      $error = 1;
      }
  } while ($error); # ||
#     ($data =~ /^\s*$/ && ($option<GATEWAY_OPTION || $option>DNS_OPTION)));
  # options 4 and 5 are optiona
  
  if ($option == LANGUAGE_OPTION)
    {
      $data = $locale_list[$data];
    }
  return $data;
}#doPrompt


sub execute {
	my $script = shift;
	my ($stdout, $stderr);
	my $rc = runit(["$script_path/$script"], \$stdout, \$stderr);
	print $stdout, $stderr;
	return $rc;
}#execute

sub continue {
  print "\n", getFormattedMessage(MSG_PRESS_ENTER);
  my $junk;
  $junk = <STDIN>;
}#continue

sub printTitle {
  #system("clear");
  print $title . "\n\n";
}#printTitle

sub run ($) {
  my ($cmd) = @_;
  my ($rc,$stdout, $stderr);
  $rc = runit([$cmd], \$stdout, \$stderr);
}

sub runOrDie ($) {
  my ($cmd) = @_;
  my ($rc,$stdout, $stderr);
  $rc = runit([$cmd], \$stdout, \$stderr);
  if ($rc) {
    print $cmd, "\n";
    print $stdout, "\n";
    print $stderr, "\n";
    exit(1);
  }
}
