#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# telnetd.pl - turns telnet on and off  for a given machine
#
# Bernd Horn <bhorn@us.ibm.com> (31 August 2000)
#

@telnetd_opts = ("disable", "enable", "status");
$telnetd_usage = "telnetd.pl --disable\ntelnetd.pl --enable\telnetd.pl --status\n";
use ASM::Common;
commonInit(\@telnetd_opts, $telnetd_usage);
setFacility('syscfg');

use ASM::Exec;
use Fcntl;
use DB_File;


if ($opt_enable) {
	&turnTelnetOnOff("enable");
	&reReadInetd();
} elsif($opt_disable) {
	&turnTelnetOnOff("disable");
	&reReadInetd();
} elsif($opt_status) {
	$return_Telnetstat = &getTelnetOnOff();
	print $return_Telnetstat . "\n";
} else {
	&logUsage();
}#if---else

exit (0);


sub getTelnetOnOff {

	my $telnet_state;
	openOrDie(INETDCONF, "<$inetd_file");

	while (<INETDCONF>) {

		if (/^#+telnet\s+/)
		{
		    $telnet_state = "OFF";
		    last;
                }
		elsif (/^telnet\s+/)
		{
		    $telnet_state = "ON";
		    last;
		} #if


	} #while

	close(INETDCONF);

	return $telnet_state;

} #getTelnetOnOff


sub turnTelnetOnOff {

	my $option = shift(@_);

	# open /etc/inetd.conf for read/write to edit telnet line
#	tie (@file, 'DB_File', "$inetd_file", O_RDWR, 0644, $DB_RECNO)
#        || die "can't update $inetd_file: $!";

	if (!tie (@file, 'DB_File', $inetd_file, O_RDWR, 0644, $DB_RECNO)) {
	  error(MSG_COULD_NOT_OPEN_FILE, $inetd_file);
	  exit(5);
	}
	my $i = 0;
        
	while (defined $file[$i]) {

        	$_ = $file[$i];

		#find the telnet line
		if (/^#*telnet\s+/){
	        	last;
        	}
        	$i++;

	} #while


	if ($option eq "enable") {
		$file[$i] = 'telnet	stream	tcp	nowait	root	/usr/sbin/tcpd	in.telnetd -h';
		audit(MSG_ENABLED_TELNET);
	} elsif ($option eq "disable") {
		$file[$i] = '#telnet	stream	tcp	nowait	root	/usr/sbin/tcpd	in.telnet -h';
		audit(MSG_DISABLED_TELNET);
	} else {
		return (3);
	}


	untie @file;

} #turnTelnetOnOff

# reload the inetd.conf file with changes
sub reReadInetd {
  # run_inet is something like "/etc/rc.d/init.d/inet reload" 
  # (see Common.pm)

  my ($rc,$stdout,$stderr);
  $rc = runit([$run_inet], \$stdout, \$stderr);
  if ($rc) {
    trace("error reloading inetd: rc=$rc\n$stdout\$stderr");
    error(MSG_ERROR_RELOADING_INETD);
    exit(6);
  }

} #reReadInetd

