#!/usr/bin/perl 
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# updateMaillist.pl
#
# Adds, edits, or deletes a Mail List.
#
# Usage:
#   updateMaillist.pl --name=<list name> --add|--edit|--delete --passwd=<password> --owner=<owner's email address with defaut of majordomo-owner>]
#                     [--addrs=<comma seperated list of email addresses with default of owner address>]
#                     [--addrfile=<file containing the new list of email addresses>
#                     [--subscribe]  [--anypost]
#                     [--help]
#   When the delete option is used, the owner and addrs options are ignored.

use ASM::Exec;
use ASM::Common;

my (@eSP_BIN, $rc, $stdout, $stderr, @opts, $updateMaillist_usage);
$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_mailmgr.pl";

#  Parse the arguments
@opts = ("name=s", "add", "edit", "delete", "passwd=s", "owner=s", "subscribe", "anypost", "addrs=s", "addrfile=s");
$updateMaillist_usage = "--name=xxx --add|--delete|--edit [--passwd=xxx] [--owner=xxx\@yyy] [--addrs=xxx\@yyy[,xxx\@yyy]] [--addrfile=xxx] [--subscribe] [--anypost]";

commonInit(\@opts, $updateMaillist_usage);

setFacility('sysuser');

use vars qw/ $opt_help $opt_name $opt_add $opt_delete $opt_passwd /;

# Get command line options and check 'em out to see if they're okay.

&GetOptions(@opts);

if ($Getopt::Long::error){  # There was an error in parsing the arguments
  exit 2;
}

if (!$opt_name || (!$opt_add && !$opt_edit && !$opt_delete) || $opt_help)  {
  printUsage();
}

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;

   print "$exitmsg\n";
   trace("$exitmsg");
   if ($exitrc != 0)
   {
      trace("Update Failed. updateMaillist.pl exiting with rc=" . $exitrc);
   }
   else
   {
      trace(getFormattedMessage(MSG_UPDATE_MAILLIST_COMPLETED, $opt_name));
   }
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.  Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

audit(MSG_UPDATE_MAILLIST_STARTED);

if ($opt_delete)
{
   trace(getFormattedMessage(MSG_DELETE_MAIL, $opt_name));
   if (&update_alias_maillist($opt_name, "delete"))
   {
      &cleanup_exit("Error deleting Mail List aliases.", 1);
   }
   &update_maillist_files($opt_name, "delete");
}
else
{
   #  Add a new Mail List or modify an existing one.
   if ($opt_add)
   {
      trace(getFormattedMessage(MSG_ADD_MAIL, $opt_name));
      if (&update_maillist_files($opt_name, "add"))
      {
         trace(getFormattedMessage(MSG_ADD_MAILLIST_FAILED, $opt_name));
         &cleanup_exit("Error adding Mail List $opt_name.", 1);
      }
   }
   else
   {
      trace(getFormattedMessage(MSG_EDIT_MAIL, $opt_name));
   }
   if (&update_alias_maillist($opt_name, "add", "majordomo-owner"))
   {
      &cleanup_exit("Error adding/modifying Mail List aliases.", 1);
   }
   if ($opt_add)
   {
      &create_maillist_config($opt_name);
      &set_maillist_approve($opt_name);
      &set_maillist_restricted($opt_name);
   }
   #  Handle Add and Edit options
   if (!$opt_delete)
   {
      if ($opt_passwd)
      {
          #  Maintain admin and approvals passwords the same.
         &set_maillist_passwd($opt_name, $opt_passwd);
      }
      if ($opt_subscribe)
      {
         &set_maillist_subscribe($opt_name);
      }
      else
      {
         &set_maillist_approve($opt_name);
      }
      if ($opt_anypost)
      {
         &set_maillist_anypost($opt_name);
      }
      else
      {
         &set_maillist_restricted($opt_name);
      }
      if ($opt_addrs)
      {
         @addrs = split(",", $opt_addrs);
         &update_maillist_addrs($opt_name, @addrs);
      }
      elsif ($opt_addrfile)
      {
         if (! -z $opt_addrfile)
         {
            &cleanup_exit("Error adding/modifying Mail List aliases.", 1);
         }
         &update_maillist_addrfile($opt_name, $opt_addrfile);
      }
   }
}

&cleanup_exit("", 0);
